#!/bin/bash

# install/esa/sbin/unconfig.sh
#
# Licensed Materials - Property of IBM
#
# Restricted Materials of IBM
#
# COPYRIGHT International Business Machines Corp. 2011,2012
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# IBM_PROLOG_END_TAG


export LOG=/opt/ibm/esa/unconfig.log
export FINALLOG=/opt/ibm/esa/log/unconfig.log
export ROOT=/opt/ibm/esa

echo "Unconfiguring ESA at `date`" > $LOG

if [[ -f "/opt/ibm/esa/locks/activated.lck" ]] ; then
  echo "Stopping ESA (if running)" >> $LOG
  ${ROOT}/bin/esactl stop
  sleep 5
else
  msg="ESA was not activated."
  echo $msg
  echo $msg >> $LOG
fi

echo "executing esa_uninstall_pre to remove error monitoring configuration" >> $LOG
${ROOT}/etc/esa_uninstall_pre

# The following directories are removed
for i in \
  ${ROOT}/workspace/.metadata \
  ${ROOT}/runtime/configuration/.settings \
  ${ROOT}/runtime/configuration/org.eclipse.core.runtime \
   ${ROOT}/runtime/configuration/org.eclipse.equinox.app \
  ${ROOT}/workspace/logs \
  ${ROOT}/ecc/data/log \
; do
  if [[ -d $i ]] ; then
    echo "...removing directory $i" >> $LOG
    rm -rf $i 2>&1 >> $LOG
  elif [[ "${i}" = "${PWD}" ]] ; then
    echo "can't remove current directory ${i}, skipping" >> $LOG
  else
    echo "...skipping ${i}, directory doesn't exist" >> $LOG
  fi
done

# The following files are removed
echo "Removing individual files" >> $LOG
for i in \
  ${ROOT}/log/* \
  ${ROOT}/equinox/logs/* \
  ${ROOT}/locks/* \
  ${ROOT}/ecc/data/GlobalConnectionManager* \
  ${ROOT}/ecc/data/*.lck \
  ${ROOT}/ecc/data/q.data \
  ${ROOT}/ecc/data/ecc[0-9]* \
  ${ROOT}/ecc/data/ecc-[0-9]* \
  ${ROOT}/ecc/data/eccReq \
  ${ROOT}/ecc/data/esatrust.keystore \
  ${ROOT}/ecc/data/esakey.keystore \
  ${ROOT}/data/IBM_ESAconfig.properties \
  ${ROOT}/ecc/data/security.properties \
  ${ROOT}/ecc/data/serviceProviderIBMLocationDefinition.xml \
  ${ROOT}/ecc/data/serviceProviderIBMLocationDefinition.txt \
  ${ROOT}/runtime/conf/webcontainer.properties \
  /tmp/esatrust.keystore \
  /tmp/esaport*.properties \
  /tmp/esatrust.keystore \
  /tmp/security.properties \
; do
  if [[ -a $i ]] ; then
    echo ...removing file $i >> $LOG
    rm -f $i 2>&1 >>$LOG
  else
    echo ...no such file as ${i}, skipped >> $LOG
  fi
done

# restore the original ssl config
SSLCONFIG_BKP=${ROOT}/runtime/conf/overrides/sslconfig.bckp
SSLCONFIG=${ROOT}/runtime/conf/overrides/sslconfig

if [[ -f $SSLCONFIG_BKP ]] ; then
echo "changing  sslconfig.bckp to sslconfig"
	mv $SSLCONFIG_BKP $SSLCONFIG
fi




# restore the original ECC configuration file
ECC_DATA_DIR=/opt/ibm/esa/ecc/data
if [[ -f $ECC_DATA_DIR/eccConnect.unconfig ]] ; then
	cp $ECC_DATA_DIR/eccConnect.properties $ECC_DATA_DIR/eccConnect.properties.bak
	cp $ECC_DATA_DIR/eccConnect.unconfig $ECC_DATA_DIR/eccConnect.properties
fi

# remove file indicating approval of license agreement
if [[ -f /opt/ibm/esa/log/la.log ]] ; then
  rm /opt/ibm/esa/log/la.log
fi

# Find ESA-related CISA configuration files and remove them
ETCCISACFG=/etc/tivoli/cce
if [[ -d $ETCCISACFG ]] ; then
    GOBACK=$PWD
    cd $ETCCISACFG
    for i in * ; do
        grep -q esa $i/* 2>/dev/null
        if [[ "$?" = "0" ]] ; then
            echo "Removing directory /etc/tivoli/cce/$i" >> $LOG
            rm -rf $i
        fi
    done
    cd $GOBACK
fi

ROOTCISACFG=/root/tivoli/cce
if [[ -d $ROOTCISACFG ]] ; then
    GOBACK=$PWD
    cd $ROOTCISACFG
    for i in * ; do
        grep -q esa $i/tivoli/uim/repository/root\#usmi/instances/* 2>/dev/null
        if [[ "$?" = "0" ]] ; then
            echo "Removing directory /root/tivoli/cce/$i" >> $LOG
            rm -rf $i
        fi
    done
    cd $GOBACK
fi

# must follow all statements that log results...
mv $LOG $FINALLOG
unset LOG

# The following files should retain their modifications:
#      /opt/ibm/esa/data/IBM_ESAconfig.properties

# These files are downloaded or derived; no need to remove:
#      /opt/ibm/esa/ecc/data/serviceProviderIBM.gzip
#      /opt/ibm/esa/ecc/data/serviceProviderIBM.xml
#      /opt/ibm/esa/ecc/data/serviceProviderIBMLocationDefinition.xml
