# IBM_PROLOG_BEGIN_TAG
#
# ESA VPL 2.0.0 install/esa/lib/ESA/Activation.pm
# Initial version for VPL
#
# Licensed Materials - Property of IBM
#
# Restricted Materials of IBM
#
# COPYRIGHT International Business Machines Corp. 2011,2012
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# IBM_PROLOG_END_TAG
package Activation;

use strict;
use warnings;
use Exporter;
use IPC::Open2;
use MIME::Base64;
use lib '/opt/ibm/esa/ecc/lib/ECC';
use lib '/opt/ibm/esa/lib/ESA';
use Constants;
use Properties;
use Utils;
use Sys::Hostname;
use Net::Domain;
use File::Find;

use vars qw(@ISA @EXPORT);
use vars qw/ $jar /;

@EXPORT = qw(getContactData setContactData);

##################################################################################

sub addErrorMonitoring() {
  return &subscribeToDiag;
}

##################################################################################

sub checkAntiPrereq() {
 return "abc";
}

##################################################################################

sub check_License {
   my ($yesno) = @_;
   my $status = -1;

   # ok if previously approved
   if ( -e Constants::LICENSE_LOG ) {
       return;
   }

   if ( $yesno eq "Y" ) {
       # if command line option signalled auto-approval, pass it along
       $status = system(Constants::LICENSE_PROG." -Y");
   } else {
       # otherwise the user has to read it.
       $status = system(Constants::LICENSE_PROG);
   }

   if ( $status != 0 ) {
       exit 1;
   }
}

##################################################################################

sub gatherLegacyPSAData() {
	&dumpEspace();

	my $classpath = &java_classpath();
	my $host = Net::Domain::domainname();

	my $cmd = getJava()." -cp $classpath com.ibm.esa.migration.psa.GWSettingsCreator $host ".Constants::LEGACY_DUMP_FILE." ".Constants::ESA_IMPORT_FILE." 1>/opt/ibm/esa/log/esa.migration.log 2>&1";
	return system($cmd);
}

##################################################################################

sub dumpEspace() {
}


##################################################################################

sub getJava() {
   # when this list changes, also change the one in esa_install_post
   # always favor 64-bit if available.
   my @java_locations=(
    '/opt/ibm/java-ppc64-80/jre/bin/java',
    '/opt/ibm/java-ppc64-70/jre/bin/java',
    '/opt/ibm/java-ppc64-71/jre/bin/java',
    '/opt/ibm/java-ppc64-60/jre/bin/java',
    '/opt/ibm/java-ppc64-50/jre/bin/java',
    '/opt/ibm/java-ppc64-80/bin/java',
    '/opt/ibm/java-ppc64-70/bin/java',
    '/opt/ibm/java-ppc64-71/bin/java',
    '/usr/lib64/jvm/jre-1.8.0-ibm/bin/java',
    '/usr/lib/jvm/jre-1.8.0-ibm/bin/java',
    '/usr/lib64/jvm/java-1.7.1-ibm-1.7.1/jre/bin/java',
    '/usr/lib64/jvm/java-1.7.1-ibm-1.7.1/bin/java',
    '/usr/lib64/jvm/java-1.7.1-ibm/jre/bin/java',
    '/usr/lib64/jvm/java-1.7.1-ibm/bin/java',
    '/opt/ibm/java-ppc64-60/bin/java',
    '/usr/lib64/jvm/jre-1_6_0-ibm/bin/java',
    '/usr/lib64/jvm/java-1_6_0-ibm-1.6.0/jre/bin/java',
    '/usr/lib64/jvm/java-1_6_0-ibm-1.6.0/bin/java',
    '/opt/ibm/java-ppc64-50/bin/java',
    '/opt/ibm/java-ppc-80/jre/bin/java',
    '/opt/ibm/java-ppc-70/jre/bin/java',
    '/opt/ibm/java-ppc-71/jre/bin/java',
    '/opt/ibm/java-ppc-60/jre/bin/java',
    '/opt/ibm/java-ppc-50/jre/bin/java',
    '/opt/ibm/java-ppc-80/bin/java',
    '/opt/ibm/java-ppc-70/bin/java',
    '/opt/ibm/java-ppc-71/bin/java',
    '/opt/ibm/java-ppc-60/bin/java',
    '/opt/ibm/java-ppc-50/bin/java',
    '/opt/ibm/java-ppc64le-80/jre/bin/java',
    '/opt/ibm/java-ppc64le-70/jre/bin/java',
    '/opt/ibm/java-ppc64le-71/jre/bin/java',
    '/opt/ibm/java-ppc64le-60/jre/bin/java',
    '/opt/ibm/java-ppc64le-50/jre/bin/java',
    '/opt/ibm/java-ppc64le-80/bin/java',
    '/opt/ibm/java-ppc64le-70/bin/java',
    '/opt/ibm/java-ppc64le-71/bin/java',
    '/opt/ibm/java-ppc64le-60/bin/java',
    '/opt/ibm/java-ppc64le-50/bin/java',
    '/usr/lib/jvm/jre/bin/java',
    '/usr/lib64/jvm/jre/bin/java'
  );
  foreach (@java_locations) {
	  if (-e $_) {
        return $_;
      }
  }
  print "No Java runtime found; activation failed; please install an IBM Java runtime.\n";
  exit(1);
}

##################################################################################

sub showESAMessage() {
	my ($set_number, $msg_number, $message) = @_;
print $message . "\n";

	# my $dspcmd = "dspmsg -s $set_number ".Constants::NLS_CATALOG." $msg_number \'$message\'";
	# select((select(STDOUT), $| = 1)[0]);
	# my $rc = system($dspcmd);
	# select((select(STDOUT), $| = 1)[0]);
	# return $rc;
    return 0;

}

##################################################################################

sub validateParameter {
  my ($flagvalue)=@_;
  $flagvalue =~ s/\s//g;
  if ("$flagvalue" eq "") {
    return 0;
    main::usage();
  }
  return 1;
}

##################################################################################

sub validate_country {
   my ($country)=@_;
   return (retrieveISOByCountry($country) eq "");
}


##################################################################################

sub validate_email($) {
 	my $email = shift;
	return $email =~/^((\w+[.-])|\w*)+\w@((\w+[.-])|\w*)+\w$/;
}

##################################################################################

sub validate_phone_number() {
 	my ($phone, $first, $last, $dash_check) = @_;
	if ( $dash_check ) {
		if ($phone =~ m/-/) {
			return (1);
		}
	}
 	my $phone_length = length($phone);
	if ( ($phone_length >= $first) && ($phone_length <= $last) ) {
		return (0);
	}
	else {
		return(1);
	}
}

##################################################################################

sub validate_au_state() {
	my ($state) = @_;
	$state = uc($state);
 	my $found = 1;
	$_ = $state; s/\s//g;
	if ($_ eq "") {
	  return $found;
	}
	open STATES, "<", Constants::AU_STATES;
	while ( (<STATES>) ) {
		my $this = $_;
		chomp $this;
		if ( $this eq $state ) {
			$found = 0;
			last;
		}
	}
	return $found;
}

##################################################################################

sub validate_us_state() {
	my ($state) = @_;
	$state = uc($state);
 	my $found = 1;
	$_ = $state; s/\s//g;
	if ($_ eq "") {
	  return $found;
	}
	open STATES, "<", Constants::US_STATES;
	while ( (<STATES>) ) {
		my $this = $_;
		chomp $this;
		if ( $this eq $state ) {
			$found = 0;
			last;
		}
	}
	return $found;
}

##################################################################################

sub validate_us_zip_format() {
 	my ($zip) = @_;
	my $good = 1;
	if ( length($zip) >= 5 ) {
		$good = 0;
	}
	return $good;
}

##################################################################################

sub validate_us_zip() {
 	my ($state, $zip) = @_;
 	$state = uc($state);
 	my $found = 1;
	open ZIPS, "<", "/opt/ibm/esa/lib/ESA/us_zipcodes";
	while ( (<ZIPS>) ) {
		my $this = $_;
		chomp $this;
		my ($new_zip, $new_state) =  split(",", $this);
		my $test_zip = substr $zip, 0, 5;
		if ( ( $new_state eq $state ) && ( $new_zip eq $test_zip ) ) {
			$found = 0;
			last;
		}
	}
	return $found;
}

##################################################################################

sub validate_ca_province() {
 	my ($prov) = @_;
 	my $found = 1;
	open PROVS, "<", "/opt/ibm/esa/lib/ESA/ca_provinces";
	while ( (<PROVS>) ) {
		my $this = $_;
		chomp $this;
		if ( $this eq $prov ) {
			$found = 0;
		}
	}
	return $found;
}

##################################################################################

sub validate_ca_zip_format() {
 	my ($zip) = @_;
	my $good = 1;
	if ( length($zip) == 6 ) {
		$good = 0;
	}

	return $good;
}

##################################################################################

sub validate_ca_zip() {
 	my ($prov, $zip) = @_;
 	my $found = 1;
	open ZIPS, "<", "/opt/ibm/esa/lib/ESA/ca_postalcodes";
	while ( (<ZIPS>) ) {
		my $this = $_;
		chomp $this;
		my ($new_zip, $new_prov) =  split(",", $this);
		my $test_zip = substr $zip, 0, 1;
		if ( ( $new_prov eq $prov ) && ( $new_zip eq $test_zip ) ) {
			$found = 0;
			last;
		}
	}
	return $found;
}

##################################################################################

sub isBOSVendorIBM() {
	my $found = 0;
	open(BOSFILE, "< @{[ Constants::BOSVENDORFILE ]}");
	LOOP: while (<BOSFILE>) {
		if (/export\s+VENDOR=IBM/) {
   			$found = 1;
			last LOOP;
		}
	}
	close(BOSFILE);
	return $found;
}

##################################################################################

sub runConnectivityTest() {
	my $result = 1;

	local (*RDR, *WTR);
	my $classpath = &java_classpath();
	#my $cmd = &getJava()." -Djava.net.preferIPv6Addresses=true -cp $classpath com.ibm.esa.activation.ConnectivityServiceTest HeartBeat";
  # comment the line below and uncomment the line above for ipv6
	my $cmd = &getJava()." -cp $classpath com.ibm.esa.activation.ConnectivityServiceTest HeartBeat";
        if ( &isLinux() ){
           $cmd= $cmd. " /opt/ibm/esa/ecc/data";
        }
	my $pid = open2(\*RDR, \*WTR, $cmd);

	while (<RDR>) {
		$result = $_;
	}

	return $result;
}

##################################################################################

sub runVerifyConnectivityTest() {
	my $result = 1;

	my $classpath = &java_classpath();
	#my $cmd = &getJava()." -Djava.net.preferIPv6Addresses=true -cp $classpath com.ibm.esa.connectivityverification.VerifyConnectivity -d /opt/ibm/esa/ecc/data -a @_";
  # comment the line below and uncomment the line above for ipv6
	my $cmd = &getJava()." -cp $classpath com.ibm.esa.connectivityverification.VerifyConnectivity -d /opt/ibm/esa/ecc/data -a @_";
    $result = system($cmd);

	return $result;
}

##################################################################################

sub esacli() {
  my $result = 0;
  my $basicdebug = Constants::DEBUG_ESAAPI_BASIC;
  my $fulldebug = Constants::DEBUG_ESAAPI_FULL;
  my $classpath = &cli_classpath();
  my $datadir = Definitions::eccRootDataDirectory;
  my $cmd = &getJava().$basicdebug.$fulldebug." -cp $classpath -Dcom.ibm.esa.api.data=".$datadir." com.ibm.esa.api.EsaCli @_";

  $result = system($cmd)/256;
  return $result;
}

##################################################################################

sub getPort() {
	my $port = 5024;
	my $classpath = &port_classPath();

	local (*RDR, *WTR);
	my $basicdebug = Constants::DEBUG_ESAAPI_BASIC;
	my $fulldebug = Constants::DEBUG_ESAAPI_FULL;

	my $cmd = &getJava().$basicdebug.$fulldebug." -cp $classpath com.ibm.esa.api.EsaApi -command=getPort -protocol=local -equinoxroot=".Constants::EQUINOXROOT;

	my $pid = open2(\*RDR, \*WTR, $cmd);

	while (<RDR>) {
		if ( $_ =~ m/=/ ) {
			(my $key, my $value) = split("=", $_);
			if ($key eq "port" ) {
				$port = $value;
				last;
			}
		}
	}

	close RDR;
	close WTR;

	return $port;
}

##################################################################################

sub setPort() {
	my ($port) = @_;
	my $status = 1;

	local (*RDR, *WTR);
	my $classpath = &port_classPath();

	my $basicdebug = Constants::DEBUG_ESAAPI_BASIC;
	my $fulldebug = Constants::DEBUG_ESAAPI_FULL;

	my $cmd = &getJava().$basicdebug.$fulldebug." -cp $classpath com.ibm.esa.api.EsaApi -command=setPort -protocol=local -newPort=$port -equinoxroot=".Constants::EQUINOXROOT;

	my $pid = open2(\*RDR, \*WTR, $cmd);

	while (<RDR>) {
		if ( $_ =~ m/=/ ) {
			(my $key, my $value) = split("=", $_);
			if ($key eq "status" ) {
				$status = $value;
				last;
			}
		}
	}

	close RDR;
	close WTR;

	return $status;
}

##################################################################################

sub getContactData() {
	my $classpath = &api_classpath();

	my $cmd = "rm -f ".Constants::ESA_IMPORT_FILE." 1>/dev/null 2>&1";
	system($cmd);

	my $basicdebug = Constants::DEBUG_ESAAPI_BASIC;
	my $fulldebug = Constants::DEBUG_ESAAPI_FULL;

	 my $status = 1;
    my $count = 0;

	$cmd = &getJava().$basicdebug.$fulldebug." -cp $classpath com.ibm.esa.api.EsaApi -command=export -resultFileName.exportFile=".Constants::ESA_IMPORT_FILE." -resultFileName.exportSuccessful=".Constants::ESA_RESULT_FILE." -exportType=properties -timeout=600000 1>>". &getApiLogFile() . " 2>&1";
	my $filename = '/opt/ibm/esa/log/activation.log';
	my $str ="";
    open(FH, '>>', $filename) or die $!;
	while ( $status != 0 && ++$count<12 ) {

 	$status = system($cmd);

	$str = join "","iteration:", $count," value of return for command export ",$status,"\n".

	print FH $str;
	sleep 10;
	}
	close(FH);
	if ( $status ne 0 ) {
		return(1);
	}

	my $rc = &parse_result_file();

	if ( $rc ne "true" ) {
		return(1);
	}

	return(0);
}

##################################################################################

sub setContactData() {
	my $classpath = &api_classpath();

	my $basicdebug = Constants::DEBUG_ESAAPI_BASIC;
	my $fulldebug = Constants::DEBUG_ESAAPI_FULL;

	 my $ret = 1;
    my $count = 0;
    my $cmd = &getJava().$basicdebug.$fulldebug." -cp $classpath com.ibm.esa.api.EsaApi -command=import -argumentFileName.importFile=".Constants::ESA_IMPORT_FILE." -timeout=600000 1>>". &getApiLogFile() . " 2>&1";
    my $filename = '/opt/ibm/esa/log/activation.log';
    my $str = "";
    open(FH, '>>', $filename) or die $!;
	while ( $ret != 0  &&  ++$count < 12 ) {

    $ret = system($cmd);

	$str = join "","iteration:", $count," value of return for command import ",$ret,"\n";

	print FH $str;
	sleep 10;

 	}
 	close(FH);
    return $ret;
}

##################################################################################

sub setIBMID() {
	my ($ibmid) = @_;
	my $classpath = &api_classpath();

	my $basicdebug = Constants::DEBUG_ESAAPI_BASIC;
	my $fulldebug = Constants::DEBUG_ESAAPI_FULL;

	my $cmd = &getJava().$basicdebug.$fulldebug." -cp $classpath com.ibm.esa.api.EsaApi -command=setIBMID -ibmid=$ibmid -timeout=600000 -resultFileName=".Constants::ESA_RESULT_FILE." 1>>" . &getApiLogFile() . " 2>&1";

	my $status = system($cmd);

	if ( $status ne 0 ) {
		return(1);
	}

	my $value = "";
	open(ESR, "< ".Constants::ESA_RESULT_FILE);
	while (<ESR>) {
		chomp;
		my $line = $_;
		if ($_ =~ m/success/) {
			(my $key, $value) = split("=", $_);
			last;
		}
	}
	close(ESR);
	$cmd = "rm -f ".Constants::ESA_RESULT_FILE." 1>/dev/null 2>&1";
	system($cmd);

	if ( $value eq "false" ) {
		return(1);
	}

	return(0);
}


##################################################################################

sub parse_result_file() {

	open(ESR, "< ".Constants::ESA_RESULT_FILE);
	my $result = <ESR>;
	chomp $result;
	close(ESR);
	my $cmd = "rm -f ".Constants::ESA_RESULT_FILE." 1>/dev/null 2>&1";
	system($cmd);
	return($result);
}

##################################################################################

my $addToPath_cpath = "";

sub addToPath
{
    return if ( $File::Find::name =~ /bundles/ );
    return if ( $File::Find::name =~ /runtime/ );
    return unless /\.jar$/;
    /$jar/ or return;
    #print "adding $jar\n";
    if ($addToPath_cpath eq "") {
	$addToPath_cpath = $File::Find::name;
    } else {
	$addToPath_cpath = $addToPath_cpath . ":".$File::Find::name;
    }
}

##################################################################################

sub buildClassPath
{
    my ($dir, @jars) = @_;
    my $currentPath = `pwd`;
    chop($currentPath);
    $addToPath_cpath = "";
    chdir($dir);
    foreach $jar (@jars) {
	  find (\&addToPath, $dir);
    }
    chdir($currentPath);
    return $addToPath_cpath;
}

##################################################################################

sub api_classpath() {
    my @jars = (
	"commons-codec",
	"commons-httpclient",
	"commons-logging",
	"com.ibm.esa.core",
	"com.ibm.esa.api",
	"com.ibm.esa.security",
	"equinoxrolemap",
	);

    return Activation::buildClassPath(Constants::ESA_INSTALL_DIR, @jars);
}

##################################################################################

sub cli_classpath() {
    my @jars = (
	"com.ibm.esa.api",
	"com.ibm.esa.security",
	"com.ibm.esa.core",
	"commons-codec",
	"commons-httpclient",
	"commons-logging",
	);

    return Activation::buildClassPath(Constants::ESA_INSTALL_DIR, @jars);
}

##################################################################################

sub java_classpath() {
	my $classpath="";

	my @ecc_jars = <\/opt\/ibm\/esa\/eclipse\/plugins\/*.jar>;

	foreach my $file (@ecc_jars) {
		$classpath="$classpath:$file";
	}

	my @esa_jars = <\/opt\/ibm\/esa\/lib\/*.jar>;

	foreach my $file (@esa_jars) {
		$classpath="$classpath:$file";
	}

	my @ecc_lib = <\/opt\/ibm\/esa\/eclipse\/plugins\/com.ibm.ecc.CommonClient*\/lib\/*.jar>;

	foreach my $file (@ecc_lib) {
		$classpath="$classpath:$file";
	}
	my @cisa_lib = <\/opt\/ibm\/esa\/eclipse\/plugins\/com.ibm.sysmgmt.core*\/classes\/*.jar>;

	foreach my $file (@cisa_lib) {
		$classpath="$classpath:$file";
	}

	return $classpath;
}

##################################################################################

sub port_classPath() {
    my @jars = (
	"commons-codec",
	"commons-httpclient",
	"commons-logging",
	"com.ibm.esa.api",
	"com.ibm.esa.security",
	);
    return Activation::buildClassPath(Constants::ESA_INSTALL_DIR, @jars);
}

##################################################################################
sub isActivated() {
	if (-e Constants::ESAACTIVATED) {
		return 0;
 	}
	else {
		return 1;
	}
}

##################################################################################

sub setActivated() {
	my $file = Constants::ESAACTIVATED;
	open(ACT, "+> $file");
	close(ACT);
        if (-e "/var/spool/cron/crontabs/esaadmin") {
		system("cronadm cron -r esaadmin");
	}

}

##################################################################################

sub isESAAlive() {
	my $cmd = "/opt/ibm/esa/bin/esa_status.sh 1>/dev/null 2>&1";

	my $timeout = 60;
	my $rc = 99;

	while (($timeout ne 0) && ($rc ne 0)) {
		$rc = system($cmd);
		$timeout = $timeout - 1;
	}

	return $rc;
}

##################################################################################
sub waitForStart(){
	my $cmd = "/opt/ibm/esa/bin/esa_status.sh 1>/dev/null 2>&1";
	my $tryNumber=6;
	while (($tryNumber ne 0) && (system($cmd) ne 0)){
		sleep 5;
		$tryNumber = $tryNumber -1;
	}
}

##################################################################################
sub waitForStop(){
	my $cmd = "/opt/ibm/esa/bin/esa_status.sh 1>/dev/null 2>&1";
	my $tryNumber=6;
	while (($tryNumber ne 0) && (system($cmd) eq 0)){
		sleep 5;
		$tryNumber = $tryNumber -1;
	}
}

##################################################################################
sub sendContact(){
	my $tryNumber=6;
	my $result = 99;
	while(($tryNumber ne 0) && ($result ne 0)){
		$result= &setContactData();
		if($result ne 0){
			sleep 5;
			$tryNumber=$tryNumber -1;
		}else{
			last;
		}
	}
	return $result;
}

##################################################################################

sub wasLegacyPSAFound() {
	if (-e Constants::LEGACY_PSA_FOUND) {
		return 0;
 	}
	else {
		return 1;
	}
}

##################################################################################

sub setLegacyPSAFound() {
	system("touch ". Constants::LEGACY_PSA_FOUND);
}

##################################################################################

sub unsetLegacyPSAFound() {
	system("rm -f ". Constants::LEGACY_PSA_FOUND);
}

##################################################################################

sub retrieveCountryByISO() {
	my ($iso_code) = @_;
	my $entry = "";
	my $cmd = "";
	if ( $iso_code ne "" ) {
	    $cmd = getJava() . " -cp /opt/ibm/esa/lib/ISOCountryManager.jar ISOCountryManager retrieveCountryByISO $iso_code";
	    #print "cmd = " . $cmd . "\n";
	    $entry = `$cmd`;
	    chomp $entry;
	}

	return $entry;
}

##################################################################################

sub retrieveISOByCountry() {
	my ($country) = @_;
	my $entry = "";
	my $cmd = "";
	$country = uc($country);
	if ( $country ne "" ) {
	    $cmd = getJava() . " -cp /opt/ibm/esa/lib/ISOCountryManager.jar ISOCountryManager retrieveISOByCountry \"$country\"";
	    $entry = `$cmd`;
	    chomp $entry;
	}

	return $entry;
}


##################################################################################

sub mkSRC() {
   if (&isLinux()){
     &addToRC();
     return 0;
   }else{
	my $mkssys_result = 99;
	my $lsitab_result = 99;
	my $mkitab_result = 0;

    my $srccmd = "mkssys -s ".Constants::SRC_NAME." -p ".Constants::START_CMD." -u ".Constants::SRC_USER." -G ".Constants::SRC_GROUP." -R -S -n 15 -f 9 -w 5 -i ".Constants::ESA_STDIN." -o ".Constants::ESA_STDOUT." -e ".Constants::ESA_STDERR;
    $mkssys_result = system($srccmd);

    my $lsitab_cmd = "lsitab ibmsrv > \/dev\/null 2>&1";
    $lsitab_result = system($lsitab_cmd);
    if ($lsitab_result ne 0) {
      my $mkitab_cmd = "mkitab \"ibmsrv:2:once:/usr/bin/startsrc -g ibmsrv > \/dev\/console 2>&1\" 1>\/dev\/null 2>&1";
      $mkitab_result = system($mkitab_cmd);
    }

	if (($mkssys_result ne 0) || ($mkitab_result ne 0)) {
      return 1;
	}
	else {
      return 0;
    }
  }
}

##################################################################################

sub rmSRC() {
   if (&isLinux()){
     return  &rmFromRC();
   }else{
	my $rmssys_result = 99;
	my $lssrc_result = 99;
	my $rmitab_result = 0;

	my $srccmd = "rmssys -s ".Constants::SRC_NAME;
	$rmssys_result = system($srccmd);

    my $lssrc_cmd = "lssrc -s IBM.SRVPROXY > \/dev\/null 2>&1";
    $lssrc_result = system($lssrc_cmd);
    if ($lssrc_result ne 0) {
      my $rmitab_cmd = "rmitab ibmsrv 1>\/dev/null 2>&1";
      $rmitab_result = system($rmitab_cmd);
    }
    if (($rmssys_result ne 0) || ($rmitab_result ne 0)) {
      return 1;
    }
    else {
      return 0;
    }
  }
}

##################################################################################

sub startSRC() {
   if (&isLinux()){
     return  &startLinuxRC();
   }else{
    my $srccmd = "startsrc -s ".Constants::SRC_NAME;
    return system($srccmd);
   }
}

##################################################################################

sub stopSRC() {
   if (&isLinux()){
     return  &stopLinuxRC();
   }else{
	my $srccmd = "stopsrc -c -s ".Constants::SRC_NAME;
	return system($srccmd);
  }
}

##################################################################################

sub increaseVAR() {
	my $response = "Filesystem /var not extended.\n";
	#my $free = `df -k \| grep \"/var\$\" \| awk \'{print \$3}\'`;
	#if ( $free < 100000 ) {
		#my $newsize = int((100000 - $free) / 1000)."M";
		#my $chfs = `chfs -a size=+$newsize \/var`;
		#$response = "Filesystem /var extended by roughly $newsize.\n";
	#}
	return $response;
}

##################################################################################

sub getElementData() {
    my ($data, $tag) = @_;
    if ($data =~ /<$tag>(.*)<\/$tag>/) {
        my $r = $1;
        $r =~ s/^\s+//;
        $r =~ s/\s+$//;
        return $r;
    }
}

##################################################################################

sub readContact(){
	my %contact_data;

	my $prop=new Properties;
	$prop->read(Constants::ESA_IMPORT_FILE);

	$contact_data{Constants::R_COMPANY_NAME} = $prop->get(Constants::R_COMPANY_NAME);
	$contact_data{Constants::R_CONTACT_NAME} = $prop->get(Constants::R_CONTACT_NAME);
	$contact_data{Constants::R_EMAIL_ADDRESS} = $prop->get(Constants::R_EMAIL_ADDRESS);
	$contact_data{Constants::R_PHONE_NUMBER} = $prop->get(Constants::R_PHONE_NUMBER);
	$contact_data{Constants::R_COUNTRY_OR_REGION} = $prop->get(Constants::R_COUNTRY_OR_REGION);
	$contact_data{Constants::R_LOCATION_PHONE} = $prop->get(Constants::R_LOCATION_PHONE);
	$contact_data{Constants::R_LOCATION_COUNTRY} = $prop->get(Constants::R_LOCATION_COUNTRY);
	$contact_data{Constants::LOCATION_ADDRESS1} = $prop->get(Constants::LOCATION_ADDRESS1);
	$contact_data{Constants::R_SEC_CONTACT_NAME} = $prop->get(Constants::R_SEC_CONTACT_NAME);
	$contact_data{Constants::R_SEC_CONTACT_EMAIL_ADDRESS} = $prop->get(Constants::R_SEC_CONTACT_EMAIL_ADDRESS);
	$contact_data{Constants::R_SEC_CONTACT_PHONE_NUMBER} = $prop->get(Constants::R_SEC_CONTACT_PHONE_NUMBER);
	$contact_data{Constants::LOCATION_CITY_OR_LOCALITY} = $prop->get(Constants::LOCATION_CITY_OR_LOCALITY);
	$contact_data{Constants::LOCATION_STATE_OR_PROVINCE} = $prop->get(Constants::LOCATION_STATE_OR_PROVINCE);
	$contact_data{Constants::LOCATION_ZIP_OR_POSTAL_CODE} = $prop->get(Constants::LOCATION_ZIP_OR_POSTAL_CODE);
	$contact_data{Constants::LOCATION_BUILDING} = $prop->get(Constants::LOCATION_BUILDING);
	$contact_data{Constants::SERVICE_AGENT_PORT} = $prop->get(Constants::SERVICE_AGENT_PORT);

	return %contact_data;
}

##################################################################################

sub updateESAProperties(){
	my %data=@_;
	&initializeProperties;
	my $prop=new Properties;
	$prop->read(Constants::ESA_IMPORT_FILE);
	my $key;
	foreach $key (keys %data) {
		$prop->set($key,$data{$key});
	}
	#add a direct connection to IBM if nothing else is present.
	 my $tmpVal=$prop->get(Constants::CONNECTION_TYPE_1);
        if (! defined($tmpVal) || ($tmpVal eq "0" ) ){
              $prop->set(Constants::CONNECTION_TYPE_1,"1");
        }
	$prop->write(Constants::ESA_IMPORT_FILE);
	return 0;
}

##################################################################################

sub initializeProperties(){
	if (! -f Constants::ESA_IMPORT_FILE ){
		system("touch ". Constants::ESA_IMPORT_FILE);
	}
}

##################################################################################
sub isLinux(){
  my $isLinux=0;
  my $cmd = "uname -s";
  my $pid = open2(\*RDR, \*WTR, $cmd);
  while (<RDR>) {
        chomp($_);
        if ($_ eq "Linux" ) {
           $isLinux = 1;
         }
        }
  return $isLinux;

}
sub addToRC(){
 if ( -f Constants::SYSTEMCTL ){
 	system(Constants::ADD_UNIT_FILE);
 	system(Constants::CHMOD_ESACTL_UNILT_FILE);
 	system(Constants::ENABLE_SYSTEMD_ESACTL);
 	return 0;
 }
 if ( -f Constants::RC_ESA_D ){
   unlink(Constants::RC_ESA_D_LINK);
   if (symlink(Constants::RC_ESA_D,Constants::RC_ESA_D_LINK )==0){
      print("Failed to create the link". Constants::RC_ESA_D_LINK."\n");
   }
   my $cmd= Constants::CHKCONFIG . " --add " . Constants::ESACTL . " --level " . Constants::RUNLEVELS . " >>". Constants::ESA_STDOUT . " 2>>" . Constants::ESA_STDERR;
   system($cmd);
 }

}
sub rmFromRC(){
  if ( -f Constants::SYSTEMCTL ){
    system(Constants::SYSTEMCTL_STOP_ESACTL);
 	system(Constants::DISENABLE_SYSTEMD_ESACTL);
 	system(Constants::RM_UNIT_FILE);
 	return 0;
 }
   system(Constants::CHKCONFIG . " --del " . Constants::ESACTL . " >>". Constants::ESA_STDOUT . " 2>>" . Constants::ESA_STDERR );
   if( unlink (Constants::RC_ESA_D_LINK)== 0){
     print("Failed to remove the link ". Constants::RC_ESA_D_LINK . "\n");
  }
}
sub startLinuxRC(){
 if ( -f Constants::SYSTEMCTL ){
 	return system(Constants::SYSTEMCTL_START_ESACTL);
 } else {
 my $start=" force-start";
 my $cmd=Constants::RC_ESA_D . $start;
 return system($cmd);
}

}
sub stopLinuxRC(){
 if ( -f Constants::SYSTEMCTL ){
 	return system(Constants::SYSTEMCTL_STOP_ESACTL);
 }else {
 my $stop=" stop";
 my $cmd=Constants::RC_ESA_D . $stop;
 return system($cmd);

}
}

sub getApiLogFile(){
 my $linuxApiLog="/opt/ibm/esa/log/api.log";
 my $aixApiLog="/var/esa/log/api.log";
 if (&isLinux()) {
   return $linuxApiLog;
 }else{
   return $aixApiLog;
 }
}
sub exists{
 my $cmd=$_[0];
 my $checkcmd=Constants::SERVICELOG_NOTIFY ." --list --command ".$cmd . " >>". Constants::ESA_STDOUT . " 2>>" . Constants::ESA_STDERR;
  system($checkcmd);
  if ($? == 0 ){
     1;
  }else{
     0;
  }
}
sub addNotify{
  my $cmd=$_[0];
  my $serviceability=$_[1];
  my $format=$_[2];
 # my $exec_cmd=Constants::SERVICELOG_NOTIFY. " --add --command=". $cmd ." --serviceable=".$serviceability. " --method=".$format . " >>". Constants::ESA_STDOUT . " 2>>" . Constants::ESA_STDERR;
   my $exec_cmd=Constants::SERVICELOG_NOTIFY. " --add --command=". $cmd . "  --match='serviceable=".$serviceability."'" ." --method=".$format . " >>". Constants::ESA_STDOUT . " 2>>" . Constants::ESA_STDERR;
  system($exec_cmd);
}
sub removeNotify{
  my $cmd=$_[0];
  my $exec_cmd=Constants::SERVICELOG_NOTIFY. " --remove --command=". $cmd . " >>". Constants::ESA_STDOUT . " 2>>" . Constants::ESA_STDERR;
  system($exec_cmd);
}
sub subscribeToDiag{
  if ( &exists(Constants::ESA_DIAG_CMD)==0 ){
     #using the old API for now. --match='serviceable=1'
     &addNotify ( Constants::ESA_DIAG_CMD,1, Constants::PAIRS_STDIN);
   }else{
     printf("ESA is already subscribed to diagnostics...\n");
  }
}
sub removeSubscription {
  if ( &exists(Constants::ESA_DIAG_CMD)==0 ){
     printf ("ESA not subsribed to diagnostics.... \n");
   }else{
     &removeNotify(Constants::ESA_DIAG_CMD);
  }
}
sub getFirewalldCmd(){
  my $cmd="/bin/".Constants::FIREWALL_CMD;
  if ( -e $cmd ){
    return $cmd;
  }
  $cmd="/usr/bin/".Constants::FIREWALL_CMD;
  if ( -e $cmd ){
    return $cmd;
  }
  return Constants::FIREWALL_CMD;
}
sub getIptablesCmd(){
  my $cmd="/sbin/".Constants::IPTABLES;
  if ( -e $cmd ){
    return $cmd;
  }
  $cmd="/usr/sbin/".Constants::IPTABLES;
  if ( -e $cmd ){
    return $cmd;
  }
  return Constants::IPTABLES;
}
sub getIptablesSaveCmd(){
  my $cmd="/sbin/".Constants::IPTABLES_SAVE;
  if ( -e $cmd ){
    return $cmd;
  }
  $cmd="/usr/sbin/".Constants::IPTABLES_SAVE;
  if ( -e $cmd ){
    return $cmd;
  }
  return Constants::IPTABLES_SAVE;
}
sub getIptablesRestoreCmd(){
  my $cmd="/sbin/".Constants::IPTABLES_RESTORE;
  if ( -e $cmd ){
    return $cmd;
  }
  $cmd="/usr/sbin/".Constants::IPTABLES_RESTORE;
  if ( -e $cmd ){
    return $cmd;
  }
  return Constants::IPTABLES_RESTORE;
}
sub checkFirewallRule(){
  my ($port1)=@_;
  my $fw_status=getFWStatus();
  my $cmd1;
  if($fw_status eq Constants::FW_RHEL7_ACTIVE){
	  $cmd1=getFirewalldCmd(). "  --list-ports |grep ". $port1." >/dev/null";
  }else{
	  $cmd1=getIptablesCmd(). " -nL |grep ACCEPT|grep ". $port1." >/dev/null";
  }
  system($cmd1);
  my $rc=($? >> 8);
  return $rc;
}

sub openFirewall(){
  my ($port1)=@_;
  my $fw_status=&getFWStatus();
  if ( $fw_status eq Constants::FW_NOT_ACTIVE){
     print(Constants::MSG_FIREWALL_NOT_ACTIVE."\n");
     return Constants::CMD_FAILED;
   }
   if ( $fw_status eq Constants::FW_SUSE2_ACTIVE ){
      &addPortToSLESFW2($port1);
      &restartSLESFW2();
   }elsif ( $fw_status eq Constants::FW_RHEL7_ACTIVE ){
       my $cmd=getFirewalldCmd()." --zone=public --add-port=".$port1."/tcp --permanent >/dev/null";
       system($cmd);
       $cmd=getFirewalldCmd()." --zone=public --add-port=".$port1."/udp --permanent >/dev/null";
       system($cmd);
       my $rc=($? >> 8);
       if ($rc == 0 ){
	       $cmd=getFirewalldCmd()." --reload >/dev/null";
	       system($cmd);
	       $rc=($? >> 8);
	       return $rc;
       }
   }else{
      my $cmd=getIptablesCmd()." -I INPUT  -p tcp --dport ".$port1." -j ACCEPT";
      system($cmd);
      my $rc=($? >> 8);
      &saveIptables();
      return $rc;
   }
}
sub unopenFirewall(){
  my ($port1)=@_;
  my $fw_status=&getFWStatus();
  if ( $fw_status eq Constants::FW_NOT_ACTIVE){
    print(Constants::MSG_FIREWALL_NOT_ACTIVE."\n");
    return Constants::CMD_FAILED;
  }
  if ( $fw_status eq Constants::FW_SUSE2_ACTIVE ){
     &removePortFromSLESFW2($port1);
     &restartSLESFW2();
  }elsif ( $fw_status eq Constants::FW_RHEL7_ACTIVE ){
       my $cmd=getFirewalldCmd()." --zone=public --remove-port=".$port1."/tcp --permanent >/dev/null";
       system($cmd);
       $cmd=getFirewalldCmd()." --zone=public --remove-port=".$port1."/udp --permanent >/dev/null";
       system($cmd);
       my $rc=($? >> 8);
       if ($rc == 0 ){
	       $cmd=getFirewalldCmd()." --reload >/dev/null";
	       system($cmd);
	       $rc=($? >> 8);
	       return $rc;
       }
   }else{
    my $cmd=getIptablesCmd()." -D INPUT -p tcp --dport ".$port1." -j ACCEPT >/dev/null 2>&1";
    system($cmd);
    my $rc=($? >> 8);
    if ( $rc ne 0 ){
      $cmd=getIptablesCmd()." -nL --line-number |grep ".$port1. "|awk '{print \$1}'|xargs ".getIptablesCmd()." -D INPUT >/dev/null 2>&1";
      system($cmd);
      $rc=($? >> 8);
    }
    &saveIptables();
    return $rc;
  }
}

sub getFWStatus(){
 my $ret=Constants::FW_NOT_ACTIVE;
 my $rc;
 if ( -f Constants::SUSE_FIREWALL_2 ) {
  system( Constants::SUSE_FIREWALL_2 . " " . Constants::STATUS . " >/dev/null 2>&1" );
  $rc=$?;
  if ($rc eq 0 ){
    $ret=Constants::FW_SUSE2_ACTIVE;
  }
 }
 if ( $ret eq Constants::FW_NOT_ACTIVE ) {
  system( Constants::SERVICE_BIN . " " . Constants::FIREWALLD ." ". Constants::STATUS. " >/dev/null 2>&1 " );
  $rc=$?;
  if ($rc eq 0 ){
    $ret=Constants::FW_RHEL7_ACTIVE;
  }
 }
 if ($ret eq Constants::FW_NOT_ACTIVE){
    my $cmd =Constants::SERVICE_BIN . " " . Constants::IPTABLES ." ". Constants::STATUS. " >/dev/null 2>&1 ";
    system($cmd);
    $rc=$?;
    if ($rc eq 0 ){
      $ret=Constants::FW_RHEL1_ACTIVE;
    }
 }
  if ($ret eq Constants::FW_NOT_ACTIVE){
    my $cmd = getIptablesCmd(). " -nL". " >/dev/null 2>&1 ";
    system($cmd);
    $rc=$?;
    if ($rc eq 0 ){
      $ret=Constants::FW_HMC_ACTIVE;
    }
 }
 return $ret;
}

sub restartSLESFW2(){
  if ( -f Constants::SUSE_FIREWALL_2 ) {
     system( Constants::SUSE_FIREWALL_2 . " stop >/dev/null 2>&1" );
     system( Constants::SUSE_FIREWALL_2 . " start >/dev/null 2>&1" );
   }
}
sub addPortToSLESFW2(){
my ($port)=@_;
open FILE , "<", Constants::SUSE_FW_CFG  or die $!;
open TMPFILE , ">", Constants::SUSE_FW_CFG.".tmp"  or die $!;
my $found=0;
while (<FILE>){
 my $temp= $_;
 if ($temp =~m/^FW_SERVICES_EXT_TCP/ || $temp =~m/^FW_SERVICES_EXT_UDP/){
    my @vals=split('=',$temp);
    my $key=$vals[0];
    my $value=$vals[1];
    if (&isValid($value)){
      if ($value=~m/^\"/){
        $value=substr ($value,1);
      }
    }
    if (&isValid($value)){
      if ($value=~m/\"$/){
        $value=substr ($value,0,length($value)-2);
      }
    }
    my @ports;
    if (&isValid($value)){
      @ports=split(' ', $value);
      foreach my $val (@ports){
        if ($port eq $val ){
          $found=1;
          last;
        }
       }
   }
   if ($found ne 1 ){
    my $newports="";
    my $counter=0;
    foreach my $val (@ports){
       if ($counter eq 0 ){
         $newports=$val;
         $counter=1;
       }else {
          $newports=$newports." ". $val;
       }
    }
    if ($counter eq 0 ){
      $newports=$port;
    }else{
      $newports=$newports." ". $port;
    }
   $temp=$key."=\"".$newports."\"\n";
   }
 }
 print TMPFILE $temp;
}
 #out of the while loop
 close FILE;
 close TMPFILE;
 if ($found eq 1 ){

  unlink Constants::SUSE_FW_CFG.".tmp";
}else{
 rename (Constants::SUSE_FW_CFG, Constants::SUSE_FW_CFG.".org");
 rename (Constants::SUSE_FW_CFG.".tmp", Constants::SUSE_FW_CFG);
}
}
sub isValid(){
  my ($src)=@_;
  if (defined($src)){
    if (length($src) eq 0 ){
      return 0;
    }else{
      return 1;
    }
  }else{
    return 0;
  }
}
sub removePortFromSLESFW2(){
  my ($port)=@_;
  open FILE , "<", Constants::SUSE_FW_CFG  or die $!;
  open TMPFILE , ">", Constants::SUSE_FW_CFG.".tmp"  or die $!;
  my $found=0;
  while (<FILE>){
   my $temp= $_;
   if ($temp =~m/^FW_SERVICES_EXT_TCP/ || $temp =~m/^FW_SERVICES_EXT_UDP/){
      my @vals=split('=',$temp);
      my $key=$vals[0];
      my $value=$vals[1];
      if (&isValid($value)){
        if ($value =~m/^\"/){
          $value=substr ($value,1);
        }
      }
      if (&isValid($value)){
        if ($value =~m/\"$/){
          $value=substr ($value,0,length($value)-2);
        }
      }
      my @ports;
      if (&isValid($value)){
        @ports=split(' ', $value);
        foreach my $val (@ports){
          if ($port eq $val ){
            $found=1;
           }
         }
      }
     if ($found ne 0 ){
      my $newports="";
      my $counter=0;
      foreach my $val (@ports){
         if ($val ne $port){
           if ($counter eq 0 ){
              $newports=$val;
              $counter=1;
            }else {
              $newports=$newports." ". $val;
            }
         }
      }
     $temp=$key."=\"".$newports."\"\n";
     }
   }
   print TMPFILE $temp;

  }
 #out of the while loop
   close FILE;
   close TMPFILE;
   if ($found eq 0 ){
    unlink Constants::SUSE_FW_CFG.".tmp";
  }else{
   rename (Constants::SUSE_FW_CFG, Constants::SUSE_FW_CFG.".org");
   rename (Constants::SUSE_FW_CFG.".tmp", Constants::SUSE_FW_CFG);
  }
}

sub saveIptables(){
  my $ret=0;
  my $tmp_file="/tmp/i.p.123-whwhwhw.tmp";
  my $cmd=getIptablesSaveCmd()." >".$tmp_file;
  system($cmd);
  $ret=$?;
  if ($ret eq 0 ){
    $cmd=getIptablesRestoreCmd()." <".$tmp_file;
    system($cmd);
    $ret= $? >> 8;
  }else{
    print("Failed to save the iptables rules\n");
    $ret=2;
  }
  unlink($tmp_file);

  $cmd =Constants::SERVICE_BIN . " " . Constants::IPTABLES ." ". Constants::SAVE. " >/dev/null 2>&1 ";
  system($cmd);
  return $ret;

}


1;
