# IBM_PROLOG_BEGIN_TAG
#
# serviceable.event.provider 1.0.0
#
# Licensed Materials - Property of IBM
#
# Restricted Materials of IBM
#
# COPYRIGHT International Business Machines Corp. 2014,2014
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# IBM_PROLOG_END_TAG
'''
Created on Apr 17, 2014
@author: Chen Shao Jie
'''
import time
import NotificationEntry
import ApplicationConstants
import threading
import os
import logging.config
#from ConfigParser import ConfigParser, NoOptionError, NoSectionError
import configparser

class ApplicationUtils:

    def tracelogger(self):
        logging.config.fileConfig(ApplicationConstants.LOG4P_PATH)
        _tracelogger = logging.getLogger("trace")
        return _tracelogger

    def activitylogger(self):
        logging.config.fileConfig(ApplicationConstants.LOG4P_PATH)
        _activitylogger = logging.getLogger("activity")
        return _activitylogger

    def getSystemInfo(self, logEntry):
        if len(self._sysvalue) > 0:
            self._sysType = self._sysvalue[3]
            self._sysModel = self._sysvalue[1]
            self._sysSerialNumber = self._sysvalue[2]
            self._sysFirmware = self._sysvalue[8]
            self._VMID = self._sysvalue[6]
            self._sysOSName = self._sysvalue[5]
            self._sysOSType = self._sysvalue[5]
            self._sysOSVersion = self._sysvalue[4]
            self._agentHostName = self._sysvalue[0]
            self._agentHostIP = self._sysvalue[7]

        self._agentName = ApplicationConstants.VERSION
        self._agentType = ""
        self._trapAppID = ApplicationConstants.APPLICATION_ID
        self._trapDateTime = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time()))
        if logEntry.action.strip() == ApplicationConstants.REQ_SERVICE_ACTION_AND_CALL_HOME.replace(":", "").strip():
            self._actionFlg = "2"
        else:
            self._actionFlg = "1"

        self._extendedErrorData = ""
        self._addtionalDetails = ""
        self._seTestFlg = ApplicationConstants.TEST_CONTENT

    def setSystemInfo(self, logEntry):
        try:
            self._fo = open(ApplicationConstants.SYS_INFO, "r")
            line = self._fo.readline()
            if line:
                # hostname:model:serial:type:OSVersion:OSName:LPAR:hostip:firmware
                self._sysvalue = line.strip().split("@@")
        except:
            # add log
            ApplicationUtils().activityLog("UNKNOW HOST", False, logEntry)
            ApplicationUtils().tracelogger().error("ApplicationUtils::SystemInfo open system info file error")
            raise Exception
        finally:
            self._fo.close()

    def getSystemId(self, host):
        systemIdFile = os.path.join(ApplicationConstants.APP_DATA_PATH, "primary-systemid")

        if not os.path.isfile(systemIdFile):
            ApplicationUtils().tracelogger().error("systemIdfile does not exist:" + systemIdFile)
            return False

        fp = open(systemIdFile , "r")

        lines = fp.readlines()
        fp.close()

        systemId = ""
        if lines:
           for line in lines:
               line = line.strip(' \t\n')
               list = line.split(",")
               if list[0] == host:
                   systemId = list[1]

        return systemId

    def activityLog(self, host, flg=True, logEntry=None):
        msg = logEntry.date + str(" ") + logEntry.time + str(" ") + logEntry.host \
        + str(" SE[") + logEntry.elog + str("]:LID[") + logEntry.lid + str("]::SRC[") \
        + logEntry.src + str("]::") + logEntry.failure + str("::") + logEntry.errorType +str("::")+logEntry.action \
        + str("::alert transmission to <") + host
        if flg:
            msg += str("> is successful")
            ApplicationUtils().activitylogger().info(msg)
        else:
            msg += str("> failed due to: <failure while sending trap>")
            ApplicationUtils().activitylogger().error(msg)

    def getTrapInfo(self, logEntry):
        ApplicationUtils.setSystemInfo(self, logEntry)
        ApplicationUtils.getSystemInfo(self, logEntry)
        notificationEntry = NotificationEntry.NotificationEntry(self._trapDateTime, self._VMID, self._trapAppID, self._agentName, self._agentType,
                                      self._agentHostIP, self._agentHostName, self._sysType , self._sysModel, self._sysSerialNumber,
                                      self._sysFirmware, self._sysOSVersion, self._sysOSName, self._sysOSType, logEntry.date + str(" ") + logEntry.time, logEntry.lid,
                                      logEntry.src, logEntry.failure, self._actionFlg, self._extendedErrorData, self._addtionalDetails, self._seTestFlg)
        return notificationEntry


    def convertStr(self, string):
        return " s '" + string + "'"

    def doSysDiscover(self):
        # collect system information before starting sysDiscover
        try:
            os.system(ApplicationConstants.CMD_GERNERATE_SYS_INFO)
            ApplicationUtils().tracelogger().info("System info was collected")
        except:
            ApplicationUtils().tracelogger().error("ApplicationUtils::startSysDiscover collection error")

    def startSysDiscover(self):
        # collect system information before starting sysDiscover
        try:
            os.system(ApplicationConstants.CMD_GERNERATE_SYS_INFO)
            ApplicationUtils().tracelogger().info("System info was collected")
        except:
            ApplicationUtils().tracelogger().error("ApplicationUtils::startSysDiscover collection error")

        _modifyTime = os.path.getmtime(ApplicationConstants.SYS_INFO)
        while True:
            time.sleep(ApplicationConstants.INTERVAL)
            if _modifyTime != os.path.getmtime(ApplicationConstants.SYS_INFO):
                try:
                    _modifyTime = os.path.getmtime(ApplicationConstants.SYS_INFO)
                    os.system(ApplicationConstants.CMD_GERNERATE_SYS_INFO)
                    ApplicationUtils().tracelogger().info("System info was collected")
                except:
                    ApplicationUtils().tracelogger().error("ApplicationUtils::startSysDiscover collection error")

    def startSendTrap(self):
        # waiting for system information collection before starting sending traps
        time.sleep(10)
        ApplicationUtils.tracelogger().info("Ready for sending traps...")
        while True:
            if ApplicationConstants.stopSendTrap:
                break
            # remove unAlive threads
            if len(ApplicationConstants.threadsNumber) <= ApplicationConstants.MAX_THREAD:
                for _thread in ApplicationConstants.threadsNumber:
                    if _thread.isAlive() == False:
                        ApplicationConstants.threadsNumber.remove(_thread)

            if ApplicationConstants.alertQueue.empty() == False:
                if len(ApplicationConstants.threadsNumber) == 0:
                    # start a new tread, and save thread ID
                    thread = _sendTrapTread()
                    thread.start()
                    ApplicationConstants.threadsNumber.append(thread)
                else:
                    # check theread id list, if list.size()<maxthreadNaumber, then start a new thread
                    if len(ApplicationConstants.threadsNumber) < ApplicationConstants.MAX_THREAD:
                        thread = _sendTrapTread()
                        thread.start()
                        ApplicationConstants.threadsNumber.append(thread)

    def stopSendTrap(self):
        # kill threads
        for _thread in ApplicationConstants.threadsNumber:
            _thread.__stop()
        ApplicationConstants.stopSendTrap = True
        return

class _sendTrapTread(threading.Thread):
    def __init__(self):
        self._logEntry = ApplicationConstants.alertQueue.get()
        threading.Thread.__init__(self)

def parseConfigForKey(section,key,default):

    try:

        config = configparser.ConfigParser()
        config.read(ApplicationConstants.CONFIGFILE)
            #TODO: Create a Constant and Use that for both
            #section and property
        syslog = config.get(section, key)
        #print(syslog)
        return syslog


    except configparser.ConfigParser().NoSectionError:

            print(default)
            return default

    except configparser.ConfigParser().NoOptionError:

            print(default)
            return default

    #def run(self):
        #self._appNotify = ApplicationNotify.ApplicationNotify(self._logEntry)
        #self._appNotify.invokeRest()

