# IBM_PROLOG_BEGIN_TAG
#
# serviceable.event.provider 1.0.0
#
# Licensed Materials - Property of IBM
#
# Restricted Materials of IBM
#
# COPYRIGHT International Business Machines Corp. 2014,2014
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# IBM_PROLOG_END_TAG
'''
Created on Apr 17, 2014
@author: Sowjanya Rao
'''
import time
import sys
import os
import subprocess
import datetime
import ApplicationConstants
from datetime import date
from LogEntry import LogEntry
from ParseErrorLog import ParseErrorLog
import configparser
from ApplicationUtils import ApplicationUtils
from ProblemCallHome import ProblemCallHome

class MonitorErrorLog:
    def tail(self, filename):
        try:
            if os.path.exists(filename) :
                popen = subprocess.Popen('tail -n 1000 -F '+filename, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True);
                while True:
                    line = popen.stdout.readline().strip()
                    if line:
                        yield line
        except:
            self.tail(filename)

    def process_matches(self, matchtext, filecreatedtime_withoutmillis,systemId):

        while True:
            line = (yield)
            try:
                #if matchtext.lower() in line.lower():
                print(matchtext)
                print(line.decode())
                print("*"*20)
                if matchtext.lower() in line.decode().lower():
                    parseErrorLog = ParseErrorLog()
                    logEntry = parseErrorLog.parseLogEntry(line.decode())
    #               email alert, etc.
                    if (logEntry != None) and (logEntry._state == 'done'):
                            print("Starting queue")
                        #if ApplicationConstants.alertQueue.qsize() <= ApplicationConstants.MAX_QUEUE_SIZE :
                            loggeddate=logEntry.date+" "+str(date.today().year)
                            loggedtime=logEntry.time
                            string_date=loggeddate+" "+loggedtime
                            print(string_date)
                            logentrytime = datetime.datetime.strptime(string_date, "%b %d %Y %H:%M:%S")
                            if (filecreatedtime_withoutmillis <= logentrytime) :
                                #ApplicationConstants.alertQueue.put(logEntry)

                                eventObj = self.create_data(logEntry, systemId, string_date)
                                ProblemCallHome().sendProblemAPICall(eventObj)

                                if not os.path.exists(ApplicationConstants.APPLICATION_LOGS) :
                                    os.makedirs(ApplicationConstants.APPLICATION_LOGS)
                                open(ApplicationConstants.LAST_ERROR_FILE, 'w')
                            else :
                                ApplicationUtils().tracelogger().info("Entry ignored")
                        #else :
                        #    ApplicationUtils().tracelogger().error("Queue reached Maximum limit:"+str(ApplicationConstants.MAX_QUEUE_SIZE));
            except:
                ApplicationUtils().tracelogger().error("Error occurred while processing the line: "+str(line.decode())+" "+str(sys.exc_info()[0]));

    def create_data(self, logEntry, systemId, ts):
        data = {ApplicationConstants.EVENT_SYSTEMID: systemId,
                       ApplicationConstants.EVENT_ID: logEntry._diag,
                       ApplicationConstants.EVENT_CLIENT_ID: logEntry._lid,
                       ApplicationConstants.EVENT_TS: ts,
                       ApplicationConstants.EVENT_ERROR_CODE: logEntry._srn,
                       ApplicationConstants.EVENT_RESOURCE_NAME: logEntry._host,
                       ApplicationConstants.EVENT_SERVICEABILITY: "callhome",
                       ApplicationConstants.EVENT_SEVERITY: "2",
                       ApplicationConstants.EVENT_DESCRIPTION: logEntry._failure,
                       ApplicationConstants.EVENT_DETAILS: logEntry._action,
                       ApplicationConstants.EVENT_STATUS: "processing",
                       ApplicationConstants.EVENT_TICKET_NO: logEntry._diag}

        return data

    def monitor(self):
        print("monitor start\n")
        config = configparser.ConfigParser()
        config.read(ApplicationConstants.CONFIGFILE)
        syslog = config.get("SYS_LOG", "SYS_LOG")
        if os.path.exists(ApplicationConstants.LAST_ERROR_FILE):
            ftimestamp = os.path.getmtime(ApplicationConstants.LAST_ERROR_FILE)
            filecreatedtime = datetime.datetime.fromtimestamp(ftimestamp)
        else :
            filecreatedtime = datetime.datetime.now()
        filecreatedtime_withoutmillis = filecreatedtime.replace(microsecond=0)
        if os.path.isfile(syslog) == False:
            syslog = ApplicationConstants.SYS_LOG

        systemId = ApplicationUtils().getSystemId("primary")
        print("systemid =" + systemId)

        for pendingEntry in ApplicationConstants.pendingList:
            print("processing pendingEntry")
            nowTime = datetime.datetime.now()
            createTime = pendingEntry._createTime
            if (nowTime-createTime).seconds>30:
                pendingEntry._state = 'done'
                loggeddate=pendingEntry.date+" "+str(date.today().year)
                loggedtime=pendingEntry.time
                string_date=loggeddate+" "+loggedtime
                eventObj = self.create_data(pendingEntry, systemId, string_date)
                ProblemCallHome().sendProblemAPICall(eventObj)
                #ApplicationConstants.alertQueue.put(pendingEntry)
                ApplicationConstants.pendingList.remove(pendingEntry)

        list_of_matches = [ApplicationConstants.REQ_SERVICE_ACTION_AND_CALL_HOME,ApplicationConstants.REQ_SERVICE_ACTION_AND_NO_CALL_HOME,ApplicationConstants.DIAG_ENCL]
        print(list_of_matches)
        #matches = [self.process_matches(string_match,filecreatedtime_withoutmillis) for string_match in list_of_matches]
        matches = [self.process_matches(string_match,filecreatedtime_withoutmillis,systemId) for string_match in list_of_matches]
        print(matches)
        for m in matches:  # prime matches
            #m.next()
            next(m)

        print("match ends\n")

        while True:
            auditlog = self.tail(syslog)
            for line in auditlog:
                for m in matches:
                    print("sending line: " + line.decode())
                    m.send(line)

if __name__ == "__main__":

#   it ensures that the log config file is changed so that the log file is created at the appropriate place
    #SynchronizConf.synchronizConf()

    print("Going monitor\n")
    MonitorErrorLog().monitor()

    sys.exit(0)
