# IBM_PROLOG_BEGIN_TAG
#
# serviceable.event.provider 1.0.0
#
# Licensed Materials - Property of IBM
#
# Restricted Materials of IBM
#
# COPYRIGHT International Business Machines Corp. 2014,2014
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# IBM_PROLOG_END_TAG
'''
Created on Apr 17, 2014
@author: Sowjanya Rao
'''
from operator import attrgetter
import datetime
from collections import defaultdict
import re
import sys
from LogEntry import LogEntry
import logging
import ApplicationConstants
from ApplicationUtils import ApplicationUtils

class ParseErrorLog:

    def parseLogEntry(self, line):
        elogRegex = "(?P<date>^\s*[\w]+\s*[\d]+)\s*(?P<time>[\d:]+)\s*(?P<host>[\w\W]+)\s*(ELOG|DIAG_ENCL)['['](?P<elog>[\d\w]+)]:\s*[L][I][D]['['](?P<lid>[a-zA-Z0-9]+)]::[S][R][C]['['](?P<src>[\w\W]+)]::(?P<failure>[\w\W]+)::(?P<errorType>[\w\W]+)::(?P<action>[\w\W]+)"
        diagRegex = "(?P<date>^\s*[\w]+\s*[\d]+)\s*(?P<time>[\d:]+)\s*(?P<host>[\w\W]+)\s*(DIAG_ENCL)['['](?P<diag>[\w\W]+)]:\s*[T][M]['['](?P<tm>[\w\W]+)]::[S][N]['['](?P<sn>[\w\W]*)]::[S][R][N]['['](?P<srn>[\w\W]+)]::(?P<failure>[\w\W]+)::(?P<action>[\w\W]+)"
        sgnameRegex = "(?P<date>^\s*[\w]+\s*[\d]+)\s*(?P<time>[\d:]+)\s*(?P<host>[\w\W]+)\s*(DIAG_ENCL)['['](?P<diag>[\w\W]+)]:\s*(?P<sgname>[\w\W]+)"

        ApplicationUtils().tracelogger().info('New Log entry detected. Parsing - started')
        elogCompiledEntry = None
        diagCompiledEntry = None
        sgnameCompiledEntry = None
        logentry = None

        try:
            elogCompiledEntry = re.compile(elogRegex)
        except:
            ApplicationUtils().tracelogger().error('Error while compiling the elogRegex')

        try:
            diagCompiledEntry = re.compile(diagRegex)
        except:
            ApplicationUtils().tracelogger().error('Error while compiling the diagRegex')

        try:
            sgnameCompiledEntry = re.compile(sgnameRegex)
        except:
            ApplicationUtils().tracelogger().error('Error while compiling the sgnameRegex')

        try:
            parsedEntry = diagCompiledEntry.search(line)
            print("search line: " + line)
        except:
            ApplicationUtils().tracelogger().error('Error while parsing the log entry using diagCompiledEntry')

        if parsedEntry != None :
            date = parsedEntry.group('date')
            time = parsedEntry.group('time')
            host = parsedEntry.group('host')
            diag = parsedEntry.group('diag')
            tm = parsedEntry.group('tm')
            sn = parsedEntry.group('sn')
            srn = parsedEntry.group('srn')
            failure = parsedEntry.group('failure')
            action = parsedEntry.group('action')
            state = 'waiting'
            createTime = datetime.datetime.now()
            logentry = LogEntry(date, time, host, "", "", srn, diag, tm, sn, srn, failure, "", action, "", state, createTime)
            ApplicationConstants.pendingList.append(logentry)
            infoString = 'date=' + str(date) + ' time=' + str(time) + 'host=' + str(host) + ' diag=' + str(diag) + ' tm=' + str(tm) + ' sn=' + str(sn) + ' srn=' + str(srn) + ' failure=' + str(failure) + ' action=' + str(action)
            ApplicationUtils().tracelogger().info(infoString)

        else:
            try:
                parsedEntry = sgnameCompiledEntry.search(line)
            except:
                ApplicationUtils().tracelogger().error('Error while parsing the log entry using sgnameCompiledEntry')
                #print "Error while parsing the log entry using sgnameCompiledEntry"

            if parsedEntry != None :
                date = parsedEntry.group('date')
                time = parsedEntry.group('time')
                host = parsedEntry.group('host')
                diag = parsedEntry.group('diag')
                sgname = parsedEntry.group('sgname')
                index1 = sgname.rfind('\'')
                sgname = sgname[15:index1]

                for diagEntry in ApplicationConstants.pendingList:
                    if (host == diagEntry._host) and (diag == diagEntry._diag):
                        if not diagEntry._state == 'done':
                            logentry = diagEntry
                            logentry._sgname = sgname
                            logentry._state = 'done'
                            ApplicationConstants.pendingList.remove(diagEntry)
                            break

            else:
                try:
                    parsedEntry = elogCompiledEntry.search(line)
                except:
                    ApplicationUtils().tracelogger().error('Error while parsing the log entry using elogCompiledEntry')
                    #print "Error while parsing the log entry using elogCompiledEntry"

                if parsedEntry != None :
                    date = parsedEntry.group('date')
                    time = parsedEntry.group('time')
                    host = parsedEntry.group('host')
                    elog = parsedEntry.group('elog')
                    diag = parsedEntry.group('elog')
                    lid = parsedEntry.group('lid')
                    src = parsedEntry.group('src')
                    failure = parsedEntry.group('failure')
                    errorType = parsedEntry.group('errorType')
                    action = parsedEntry.group('action')
                    state = 'done'
                    logentry = LogEntry(date, time, host, elog, lid, src, diag, "", "", src, failure, errorType, action, "", state, "")
                    infoString = 'date=' + str(date) + ' time=' + str(time) + 'host=' + str(host) + ' se=' + str(elog) + ' lid=' + str(lid) + ' src=' + str(src) + ' failure=' + str(failure) + ' errorType=' + str(errorType) + ' action=' + str(action)
                    ApplicationUtils().tracelogger().info(infoString)

                else:
                    ApplicationUtils().tracelogger().info("log entry is null")
                    #print "log entry is null"

        return logentry
