from readESAUrl import getEsaUrl
import ApplicationConstants
from ApplicationUtils import ApplicationUtils
import urllib.request
import ssl
import os
import json


class ProblemCallHome():
    
    def getPrimayUrl(self):
        primayUrl=""
        try:
            primayUrl = getEsaUrl()
        except Exception:
            print("An error happened when getting rest url")
            ApplicationUtils().tracelogger().error("An error happened when getting rest url")
        return primayUrl
            
    def checkSystemId(self, data):
        systemId = data.get('system.id')
        if systemId is None or systemId == '':
            ApplicationUtils().tracelogger().info("systemId is None or blank, then considering that as the local system data...")
            systemId = ApplicationUtils().getSystemId("primary")
            data['system.id'] = systemId
  
    def problemStore(self, systemId, eventId, ticketNo, status):
        lines = ""
        problemStr = systemId + "," + eventId + "," + ticketNo
        print(ApplicationConstants.APP_DATA_PATH)
        problemFile = os.path.join(ApplicationConstants.APP_DATA_PATH, ApplicationConstants.PROBLEM_EVENT_FILE)
        print(problemFile)
        if os.path.isfile(problemFile):
            fp = open(problemFile, "r")
            lines = fp.readlines()
            fp.close()

        fp = open(problemFile, "w")
        problemExist = False
        if lines:
            for line in lines:
                if problemStr in line:
                    fp.write(problemStr)
                    fp.write(",")
                    fp.write(status)
                    fp.write("\n")
                    problemExist = True
                else:
                    fp.write(line)

        if not problemExist:
            fp.write(problemStr)
            fp.write(",")
            fp.write(status)
            fp.write("\n")
    
        fp.close()

    def problemUpdateAll(self):
        lines = ""
        problemFile = os.path.join(ApplicationConstants.APP_DATA_PATH, ApplicationConstants.PROBLEM_EVENT_FILE)

        if os.path.isfile(problemFile):
            fp = open(problemFile, "r")
            lines = fp.readlines()
            fp.close()
        else:
            ApplicationUtils().tracelogger().info("Problem file {0} does not exist".format(problemFile))
            return

        fp = open(problemFile, "w")
        problemExist = False
        problemStatus = ""
        if lines:
            for line in lines:
                list = line.split(",")
                eventId = list[1]
                problemStatus = self.getProblemStatus(eventId) 
                problem = self.getProblemStatus(eventId) 
                if problem:
                    if problem['code'] == 451:
                        continue
                    else:
                        fp.write(list[0])
                        fp.write(",")
                        fp.write(list[1])
                        fp.write(",")
                        fp.write(problem[ApplicationConstants.EVENT_TICKET_NO])
                        fp.write(",")
                        fp.write(problem[ApplicationConstants.EVENT_STATUS])
                        fp.write("\n")
                else:
                    fp.write(line)

        fp.close()

    def problemUpdateSingle(self, eventId):
        lines = ""
        problemFile = os.path.join(ApplicationConstants.APP_DATA_PATH, ApplicationConstants.PROBLEM_EVENT_FILE)

        if os.path.isfile(problemFile):
            fp = open(problemFile, "r")
            lines = fp.readlines()
            fp.close()
        else:
            ApplicationUtils().tracelogger().info("Problem file {0} does not exist".format(problemFile))
            return

        fp = open(problemFile, "w")
        if lines:
            for line in lines:
                list = line.split(",")
                if eventId == list[1]:
                    problem = self.getProblemStatus(eventId) 
                    if problem:
                        if problem['code'] == 451:
                            continue
                        else:
                            fp.write(list[0])
                            fp.write(",")
                            fp.write(list[1])
                            fp.write(",")
                            fp.write(problem[ApplicationConstants.EVENT_TICKET_NO])
                            fp.write(",")
                            fp.write(problem[ApplicationConstants.EVENT_STATUS])
                            fp.write("\n")
                    else:
                        fp.write(line)
                else:
                    fp.write(line)

        fp.close()

    def getProblemStatus(self, eventId):
        primayUrl = self.getPrimayUrl()
        if(primayUrl ==""):
            ApplicationUtils().tracelogger().error("An error happened when getting ESA-URLS-USER.conf ")
            return
        
        getProblemUrl = primayUrl + "/event/" + eventId
        print(getProblemUrl)
        problem = {}
        try:
            h = {"Content-Type": "application/json"}
            httpRequest = urllib.request.Request(getProblemUrl, headers=h)
            httpRequest.get_method = lambda: "GET"

            ctx = ssl.create_default_context()
            ctx.check_hostname = False
            ctx.verify_mode = ssl.CERT_NONE

            httpResponse = urllib.request.urlopen(httpRequest, context=ctx)
            if httpResponse.getcode() == 200:
                dataJson = json.loads(httpResponse.read().decode("utf-8"))
                print(dataJson)
                ApplicationUtils().tracelogger().info("sendProblemAPICall response = {0}".format(dataJson))
                code = dataJson['status']['code']
                if (code == 200): 
                    problem[ApplicationConstants.EVENT_TICKET_NO] = dataJson['event'][ApplicationConstants.EVENT_TICKET_NO]
                    problem[ApplicationConstants.EVENT_STATUS] = dataJson['event'][ApplicationConstants.EVENT_STATUS]
                    problem['code'] = code
                if (code == 451): 
                    problem['code'] = code
                elif (code == 201) or (code == 452) or (code == 422):
                    result = dataJson['status']['message']
                    print("Error from csESA {0}".format(json.dumps(result)))
                    ApplicationUtils().tracelogger().error("getProblemStatus,error from csESA {0}".format(json.dumps(result)))
        except urllib.error.HTTPError as he:
            ApplicationUtils().tracelogger().error("An error happened when call problem API.")
            if hasattr(he, 'code'):
                print('Error code: ', str(he.code))
                ApplicationUtils().tracelogger().error('Error code: ', str(he.code))
        except urllib.error.URLError as ue:
            ApplicationUtils().tracelogger().error("An error happened when call problem API.")
            if hasattr(ue,"reason"):
                print("Reason:"+ str(ue.reason))
                ApplicationUtils().tracelogger().error("Reason:"+ str(ue.reason))
            if hasattr(ue, 'code'):
                print('Error code: ', str(ue.code))
                ApplicationUtils().tracelogger().error('Error code: '+ str(ue.code))
        except BaseException as be:
            print("An error happened when send problem API.BaseException %s" %be)
            ApplicationUtils().tracelogger().error("An error happened when call problem API.BaseException {0}".format(be))
                    
        return problem

    def sendProblemAPICall(self, data):
        print(data)
        self.checkSystemId(data)
        
        primayUrl = self.getPrimayUrl()
        if(primayUrl ==""):
            ApplicationUtils().tracelogger().error("An error happened when getting ESA-URLS-USER.conf ")
            return
        
        data = json.dumps(data).encode()
        sendProblemUrl = primayUrl+"/event"
        print(sendProblemUrl)
        try:
            h = {"Content-Type": "application/json"}
            httpRequest = urllib.request.Request(sendProblemUrl, data=data, headers=h)
            httpRequest.get_method = lambda: "POST"

            ctx = ssl.create_default_context()
            ctx.check_hostname = False
            ctx.verify_mode = ssl.CERT_NONE

            httpResponse = urllib.request.urlopen(httpRequest, context=ctx)
            if httpResponse.getcode() == 200:
                dataJson = json.loads(httpResponse.read().decode("utf-8"))
                print(dataJson)
                ApplicationUtils().tracelogger().info("sendProblemAPICall response = {0}".format(dataJson))
                code = dataJson['status']['code']
                if (code == 200): 
                    result = dataJson['event']['system.id']
                    print("sendProblemAPICall,it is successful. systemid = {0}".format(result))
                    ApplicationUtils().tracelogger().info("sendProblemAPICall,it is successful. systemid = {0}".format(result))
                    self.problemStore(dataJson['event']['system.id'],dataJson['event']['event.id'],dataJson['event']['ticket.no'],dataJson['event']['status'])
                elif (code == 201) or (code == 452) or (code == 422):
                    result = dataJson['status']['message']
                    print("Error from csESA {0}".format(json.dumps(result)))
                    ApplicationUtils().tracelogger().error("sendProblemAPICall,error from csESA {0}".format(json.dumps(result)))
        except urllib.error.HTTPError as he:
            ApplicationUtils().tracelogger().error("An error happened when call send problem API.")
            if hasattr(he, 'code'):
                print('Error code: ', str(he.code))
                ApplicationUtils().tracelogger().error('Error code: ', str(he.code))
        except urllib.error.URLError as ue:
            ApplicationUtils().tracelogger().error("An error happened when call send problem API.")
            if hasattr(ue,"reason"):
                print("Reason:"+ str(ue.reason))
                ApplicationUtils().tracelogger().error("Reason:"+ str(ue.reason))
            if hasattr(ue, 'code'):
                print('Error code: ', str(ue.code))
                ApplicationUtils().tracelogger().error('Error code: '+ str(ue.code))
        except BaseException as be:
            print("An error happened when call send problem API.BaseException %s" %be)
            ApplicationUtils().tracelogger().error("An error happened when call send problem API.BaseException {0}".format(be))
                    
        return 
        
if __name__ == "__main__":
    tempData={
       "system.id": "",
       "event.id": "1234",
       "event.client.id": "string",
       "event.original.timestamp": "Mon Jul 31 06:40:06 EDT 2016",
       "error.code": "1234",
       "resource.name": "hdisk1",
       "event.serviceability": "callhome",
       "severity": "2",
       "event-description": "string",
       "event-details": "string",
       "frus": [
            {
               "fru.name": "ABCS",
               "fru.type": "M",
               "fru.class": "string",
               "fru.serial": "BA123S",
               "fru.description": "string",
               "location.code": "string",
               "fru.ccin": "string",
               "ref.code": "string",
               "fru.priority": "2",
               "fru.probability": "3",
               "fru.procedure.number": "string",
               "original.part.number": "string",
               "replacement.part.number": "string",
               "replaced.date": "string",
               "enclosure.info": "string",
               "controlling.machine": "string",
               "Test.data": "TEST DATA",
               "additional.data": "ADDITIONAL DATA"
             }
        ],
        "eed": [
            {
                "data.description": "string",
                "data.location": "/opt/ibm/sowjanya/sample.txt"
             }
        ],
        "status": "processing",
        "ticket.no": "1234567"
     }
    ProblemCallHome().sendProblemAPICall(tempData)       
    
