# IBM_PROLOG_BEGIN_TAG
#
# serviceable.event.provider 1.0.0 
#
# Licensed Materials - Property of IBM
#
# Restricted Materials of IBM
#
# COPYRIGHT International Business Machines Corp. 2014,2014
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# IBM_PROLOG_END_TAG
'''
Created on Oct 20, 2018

@author: Shawn Du
'''

import subprocess
import sys
import os
import ssl
import json
#import urllib2
import urllib
import urllib.request
import urllib.error
import socket
import fcntl
import struct
import ApplicationConstants
#import ApplicationUtils
from ApplicationUtils import ApplicationUtils
import readESAUrl
#import ParseErrorLog
import SynchronizConf

def registerEndpoint():
    sysdiscover = subprocess.Popen(ApplicationConstants.CMD_GERNERATE_SYS_INFO, shell=True, stdout=subprocess.PIPE,
                             stderr=subprocess.STDOUT)
    sysinfo = []
    for line in sysdiscover.stdout.readlines():
        lineStr = line.decode("utf-8")
        if len(lineStr.split("=")) > 1:
            sysinfo.append(lineStr.split("=")[1][1:-1])

    hardwareObj = [{ApplicationConstants.END_PROD_HARD_INFO: "basic",
                      ApplicationConstants.END_PROD_HARD_VENDOR: sysinfo[4],
                      ApplicationConstants.END_PROD_HARD_MODEL: sysinfo[2],
                      ApplicationConstants.END_PROD_HARD_MT: sysinfo[1],
                      ApplicationConstants.END_PROD_HARD_SN: sysinfo[3]}]

    endpointObj = {ApplicationConstants.END_HOSTNAME: sysinfo[0],
                      ApplicationConstants.END_APP_OWNER: ApplicationConstants.APPLICATION_NAME,
                      ApplicationConstants.END_OS_TYPE: sysinfo[6],
                      ApplicationConstants.END_OS_ARCH: "Arch",
                      ApplicationConstants.END_PROD_HARD: hardwareObj,
                      ApplicationConstants.END_ENDPOINT_TYPE: sysinfo[5],
                      ApplicationConstants.END_ENDPOINT_ARCH: "Arch"}

    print(endpointObj)

    data = json.dumps(endpointObj).encode()

    try:
        csESAUrl = readESAUrl.getEsaUrl()
        ApplicationUtils().tracelogger().info("esaUrl = {0}".format(csESAUrl))

        restUrl = csESAUrl + '/endpoint/'

        h = {"Content-Type": "application/json"}
        httpRequest = urllib.request.Request(restUrl, data=data, headers=h)
        httpRequest.get_method = lambda: 'POST'

        ctx = ssl.create_default_context()
        ctx.check_hostname = False
        ctx.verify_mode = ssl.CERT_NONE

        httpResponse = urllib.request.urlopen(httpRequest, context=ctx)

        if httpResponse.getcode() == 200:
            #if httpResponse.info().gettype() == 'application/json':
                dataJson = json.loads(httpResponse.read().decode("utf-8"))
                print(dataJson)
                ApplicationUtils().tracelogger().info("httpResponseRead is {0}".format(json.dumps(dataJson, sort_keys=True,indent=4, separators=(',', ': '))))
                code = dataJson['status']['code']
                if (code == 200) or (code == 201) or (code == 202):
                    result = dataJson['system.id']
                    ApplicationUtils().tracelogger().info("systemid = {0}".format(result))
                    systemIdStore("primary",json.dumps(result))
                elif (code == 451) or (code == 422) or (code == 404):
                    result = dataJson['status']['message']
                    ApplicationUtils().tracelogger().error("Error from csESA {0}".format(json.dumps(result)))
                    print(json.dumps(result))

    except urllib.error.HTTPError as he:
        print(he.code)
        print(he.read())
        print(format(he))
        ApplicationUtils().tracelogger().error("urllib.error.HTTPError ({0}): {1}".format(he.errno, he.strerror))
    except urllib.error.URLError as ue:
        print(ue.reason)
        ApplicationUtils().tracelogger().error("urllib.error.URLError ({0}): {1}".format(ue.errno, ue.strerror))
    except BaseException as be:
        ApplicationUtils().tracelogger().error("BaseException {0}".format(be))
        print("BaseException %s" %be)

    #return code

def systemIdStore(hostName, systemId):
    lines = ""
    print(ApplicationConstants.APP_DATA_PATH)
    systemIdFile = os.path.join(ApplicationConstants.APP_DATA_PATH, "primary-systemid")
    print(systemIdFile)
    if os.path.isfile(systemIdFile):
        fp = open(systemIdFile, "r")
        lines = fp.readlines()
        fp.close()

    fp = open(systemIdFile, "w")
    systemExist = False
    if lines:
        for line in lines:
            fp.write(line)
            if hostName in line:
                systemExist = True

    if not systemExist:
        fp.write(hostName)
        fp.write(",")
        fp.write(systemId.replace('"', ''))
        fp.write("\n")
    
    fp.close()


if __name__ == "__main__": 

#   it ensures that the log config file is changed so that the log file is created at the appropriate place
    SynchronizConf.synchronizConf()

    registerEndpoint()
    
    sys.exit(0)
