# IBM_PROLOG_BEGIN_TAG
#
# serviceable.event.provider 1.0.0
#
# Licensed Materials - Property of IBM
#
# Restricted Materials of IBM
#
# COPYRIGHT International Business Machines Corp. 2014,2014
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# IBM_PROLOG_END_TAG

'''
Created on Sep 1, 2017
@author: Yi Qiang Wen
'''

import sys
import os
import json
import urllib2
import urllib
import socket
import fcntl
import struct
#import ApplicationNotify
#from ApplicationUtils import ApplicationUtils

def usage():
    print ("")
    print ("USAGE: sendEvent [eventData]")
    print ("")
    print (" e.g: python SendEvent.py \"{'system.id':'bb4145c8caaaba930ce2e9af60464ccd','event.id':'string','event.client.id':'string','event.original.timestamp':'2016-01-0203:04:05','error.code':'5','resource.name':'disk0','severity':'2','event-description':'Test','event.serviceability':'callhome','event-details':'string','frus':[{'fru.name':'testfru','fru.type':'R','fru.class':'','fru.serial':'','fru.description':'','location.code':'','fru.ccin':'','ref.code':'','fru.priority':'','fru.procedure.number':'','original.part.number':'','replacement.part.number':'','replaced.date':'','enclosure.info':'','controlling.machine':'','additional.data':''}],'eed':[{'data.description':'string','data.location':'string'}],'status':'processing','ticket.no':'string'}\"")
    print ("")
    sys.exit(1)


def sendEvent(data):
    try:
        csESAUrl = get_esa_url()
        url = csESAUrl + 'event/'

        h = {"Content-Type": "application/json"}
        req = urllib2.Request(url, data, headers=h)
        httpResponse = urllib2.urlopen(req)

        if httpResponse.getcode() == 200:
            if httpResponse.info().gettype() == 'application/json':
                dataJson = json.loads(httpResponse.read())
                ApplicationUtils().tracelogger().info("httpResponseRead is {0}".format(json.dumps(dataJson, sort_keys=True,indent=4, separators=(',', ': '))))
                code = dataJson['status']['code']
                if (code == 200) or (code == 201) or (code == 202):
                    result = dataJson['event']
                    print json.dumps(result)
                elif (code == 451) or (code == 422) or (code == 404):
                    result = dataJson['status']['message']
                    print json.dumps(result)

    except urllib2.HTTPError as he:
        print he.code
        print he.read()
        print format(he)
        ApplicationUtils().tracelogger().error("urllib2.HTTPError ({0}): {1}".format(he.errno, he.strerror))
    except urllib2.URLError as ue:
        print ue.code
        print ue.read()
        print format(ue)
        ApplicationUtils().tracelogger().error("urllib2.URLError ({0}): {1}".format(ue.errno, ue.strerror))
    except BaseException as be:
        ApplicationUtils().tracelogger().error("BaseException {0}".format(be))
        print "BaseException %s" %be

    return code

def get_local_ip(ifname):
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    return socket.inet_ntoa(fcntl.ioctl(
        s.fileno(),
        0x8915,  # SIOCGIFADDR
        struct.pack('256s', ifname[:15])
    )[20:24])

def get_esa_url():
    localIP = get_local_ip('eth0')
    #TODO: get csESA version from config file
    esaUrl = 'http://' + localIP + ':5023/rest/v1/'
    return esaUrl

if __name__ == "__main__":
    if len(sys.argv) != 2:
        usage()

    ref = sendEvent(sys.argv[1])
    print ref
