# IBM_PROLOG_BEGIN_TAG
#
# serviceable.event.provider 1.0.0
#
# Licensed Materials - Property of IBM
#
# Restricted Materials of IBM
#
# COPYRIGHT International Business Machines Corp. 2014,2014
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# IBM_PROLOG_END_TAG
'''
Created on Apr 17, 2014
@author: Chen Shao Jie
'''
import os, ApplicationNotify
import shutil
import ApplicationConstants
import ApplicationUtils
import configparser

def checkDir(logfile):
    try:
        path = os.path.split(logfile)
        if not os.path.exists(path[0]):
            os.makedirs(path[0])
        f = open(os.path.join(path[0], "a"), "w")
        f.close()
        os.remove(os.path.join(path[0], "a"))
    except:
        print ("have no permission")
        return False
    return True

def configreplace():
    defaultLogPath = ApplicationConstants.APPLICATION_LOGS
    isActivityDir = True
    isActivityMaxSize = True
    isActivityMaxCount = True
    isTraceDir = True
    isTraceMaxSize = True
    isTraceMaxCount = True
    config = configparser.ConfigParser()
    config.read(ApplicationConstants.CONFIGFILE)
    activitylog = config.get("ACTIVITY_LOG", "ACTIVITY_LOG")
    activitymaxsize = config.get("ACTIVITY_LOG", "MAXSIZE")
    activitymaxcount = config.get("ACTIVITY_LOG", "MAXCOUNT")
    tracelog = config.get("TRACE_LOG", "TRACE_LOG")
    tracemaxsize = config.get("TRACE_LOG", "MAXSIZE")
    tracemaxcount = config.get("TRACE_LOG", "MAXCOUNT")
    if checkDir(activitylog) == False:
        isActivityDir = False
        path = os.path.split(activitylog)
        if not os.path.exists(defaultLogPath):
            os.makedirs(defaultLogPath)
    if checkDir(tracelog) == False:
        isTraceDir = False
        if not os.path.exists(defaultLogPath):
            os.makedirs(defaultLogPath)
    if activitymaxsize.isdigit() == False:
        isActivityMaxSize = False
    if activitymaxcount.isdigit() == False or int(activitymaxcount) < 1:
        isActivityMaxCount = False
    if tracemaxsize.isdigit() == False:
        isTraceMaxSize = False
    if tracemaxcount.isdigit() == False or int(tracemaxcount) < 1:
        isTraceMaxCount = False
    try:
        fo = open(ApplicationConstants.TEMP_FILE, "a")
        fo.writelines("# log4p.conf\n")
        fo.writelines("[loggers]\n")
        fo.writelines("keys=root,trace,activity\n")
        fo.writelines("[handlers]\n")
        fo.writelines("keys=consoleHandler,traceHandler,activityHandler\n")
        fo.writelines("[formatters]\n")
        fo.writelines("keys=traceFormatter,activityFormatter\n")
        fo.writelines("[formatter_traceFormatter]\n")
        fo.writelines("format=[%(asctime)s](%(levelname)s)%(name)s : %(message)s\n")
        fo.writelines("[formatter_activityFormatter]\n")
        fo.writelines("format= %(message)s\n")
        fo.writelines("[logger_root]\n")
        fo.writelines("level=INFO\n")
        fo.writelines("handlers=consoleHandler\n")
        fo.writelines("[logger_trace]\n")
        fo.writelines("level=INFO\n")
        fo.writelines("handlers=traceHandler\n")
        fo.writelines("qualname=trace \n")
        fo.writelines("propagate=0\n")
        fo.writelines("[logger_activity]\n")
        fo.writelines("level=INFO\n")
        fo.writelines("handlers=activityHandler\n")
        fo.writelines("qualname=activity \n")
        fo.writelines("propagate=0\n")
        fo.writelines("[handler_traceHandler]\n")
        fo.writelines("class=handlers.RotatingFileHandler\n")
        fo.writelines("level=INFO\n")
        fo.writelines("formatter=traceFormatter\n")

        if isTraceDir:
            line = "args=('" + tracelog + "', 'a', "
        else:
            line = "args=('" + defaultLogPath + "/trace.log"  + "', 'a', "
        if isTraceMaxSize:
            line += tracemaxsize
        else:
            line += "2048000"
        if isTraceMaxCount:
            line += "," + str(int(tracemaxcount) - 1) + ")"
        else:
            line += ",2)"
        fo.writelines(line + "\n")

        fo.writelines("[handler_activityHandler]\n")
        fo.writelines("class=handlers.RotatingFileHandler\n")
        fo.writelines("level=INFO\n")
        fo.writelines("formatter=activityFormatter\n")

        if isActivityDir:
            line1 = "args=('" + activitylog + "', 'a', "
        else:
            line1 = "args=('" + defaultLogPath + "/activity.log" + "', 'a', "
        if isActivityMaxSize:
            line1 += activitymaxsize
        else:
            line1 += "2048000"
        if isActivityMaxCount:
            line1 += "," + str(int(activitymaxcount) - 1) + ")"
        else:
            line1 += ",2)"
        fo.writelines(line1 + "\n")

        fo.writelines("[handler_consoleHandler]\n")
        fo.writelines("class=StreamHandler\n")
        fo.writelines("level=INFO\n")
        fo.writelines("formatter=traceFormatter\n")
        fo.writelines("args=(sys.stdout,)\n")
    except:
        return False
    finally:
        fo.close()

    shutil.copyfile(ApplicationConstants.TEMP_FILE, ApplicationConstants.LOG4P_PATH)
    os.remove(ApplicationConstants.TEMP_FILE)
    return True

def readVersion():
    versionPath = os.path.join(ApplicationConstants.APPLICATION_ROOT,"version.properties")
    fop = open(versionPath, "r")
    try:
        while True:
            line = fop.readline()
            line.replace("Version: ","")
            if line:
                ApplicationConstants.VERSION = line.replace("Version: ","")
                break
            else:
                break
    except:
        # add log
        ApplicationUtils().tracelogger().error("readVersion::open version file error")
    finally:
        fop.close()

def synchronizConf():
    readVersion()
    if configreplace():
        ApplicationUtils.ApplicationUtils().tracelogger().info("configure file was synchronized")
        return True
    else:
        ApplicationUtils.ApplicationUtils().tracelogger().error("configure file synchronized failure due to wrong parameters")
        return False


if __name__ == "__main__":
    if synchronizConf():
        print ("configure file was synchronized")
    readVersion()
