# IBM_PROLOG_BEGIN_TAG
#
# serviceable.event.provider 1.0.0
#
# Licensed Materials - Property of IBM
#
# Restricted Materials of IBM
#
# COPYRIGHT International Business Machines Corp. 2014,2014
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# IBM_PROLOG_END_TAG
'''
Created on Ago 25, 2017
@author: Miguel Angel Guzman Pantoja
'''
#import urllib2
import urllib
import urllib.request
import urllib.error
import json
import ssl
from Contact import Contact
from ApplicationUtils import ApplicationUtils
import SynchronizConf
import readESAUrl

class UpdateSettings:

    #Fields for security information
    SECURITY = "security"
    TOKENS = "tokens"

    #Fields for contact information
    CONTACTS = "contacts"
    CONTACT_TYPE = "contact.type"
    PRIMARY = "primary"
    SECONDARY = "secondary"
    EMAIL = "email"
    EMAIL_ALT = "email.alt"
    PHONE = "phone"
    PHONE_ALT = "phone.alt"
    PHONE_EXTENSION = "phone.extension"
    PHONE_ALT_EXTENSION = "phone.alt.extension"
    NAME = "name"
    NAME_ALT = "name.alt"

    def updateContact(self, esaURL, token, contact):
        if contact is None or token is None or token == "":
           print("Invalid parameter; token or contact should not be null.")

        tokenObj = {UpdateSettings.TOKENS:token}

        contactPriObj = {UpdateSettings.CONTACT_TYPE : UpdateSettings.PRIMARY,
                      UpdateSettings.NAME: contact.primaryName,
                      UpdateSettings.NAME_ALT: contact.primaryNameAlt,
                      UpdateSettings.EMAIL: contact.primaryEmail,
                      UpdateSettings.EMAIL_ALT: contact.primaryEmailAlt,
                      UpdateSettings.PHONE: contact.primaryPhone,
                      UpdateSettings.PHONE_EXTENSION: contact.primaryPhoneExt,
                      UpdateSettings.PHONE_ALT: contact.primaryPhoneAlt,
                      UpdateSettings.PHONE_ALT_EXTENSION: contact.primaryPhoneAltExt}

        contactSecObj = {UpdateSettings.CONTACT_TYPE: UpdateSettings.SECONDARY,
                         UpdateSettings.NAME: contact.secundaryName,
                         UpdateSettings.NAME_ALT: contact.secundaryNameAlt,
                         UpdateSettings.EMAIL: contact.secundaryEmail,
                         UpdateSettings.EMAIL_ALT: contact.secundaryEmailAlt,
                         UpdateSettings.PHONE: contact.secundaryPhone,
                         UpdateSettings.PHONE_EXTENSION: contact.secundaryPhoneExt,
                         UpdateSettings.PHONE_ALT: contact.secundaryPhoneAlt,
                         UpdateSettings.PHONE_ALT_EXTENSION: contact.secundaryPhoneAltExt}
        contactArray = [contactPriObj, contactSecObj]

        updateConctact = {UpdateSettings.SECURITY: tokenObj,
                          UpdateSettings.CONTACTS: contactArray}

        restURL = esaURL + "/settings/contact"

        ctx = ssl.create_default_context()
        ctx.check_hostname = False
        ctx.verify_mode = ssl.CERT_NONE

        data = json.dumps(updateConctact).encode()
        httpRequest = urllib.request.Request(restURL, data=data)
        httpRequest.add_header("Content-Type", 'application/json')
        httpRequest.get_method = lambda: 'POST'
        dataJson = ""

        try:
            httpResponse = urllib.request.urlopen(httpRequest, context=ctx)

            print("httpResponseCode is {0}".format(httpResponse.getcode()))
            if httpResponse.getcode() == 200:
                # success
                #if httpResponse.info().gettype() == 'application/json':
                    dataJson = json.loads(httpResponse.read().decode("utf-8"))

        except urllib.error.HTTPError as he:
            ApplicationUtils().tracelogger().error("urllib.error.HTTPError ({0}): {1}".format(he.errno, he.strerror))
        except urllib.error.URLError as ue:
            ApplicationUtils().tracelogger().error("urllib.error.URLError ({0}): {1}".format(ue.errno, ue.strerror))
        except BaseException as be:
            ApplicationUtils().tracelogger().error("BaseException {0}".format(be))

        return dataJson

if __name__ == "__main__":

    #   it ensures that the log config file is changed so that the log file is created at the appropriate place
    SynchronizConf.synchronizConf()

    contacts = Contact()

    contacts.primaryName = "PrimatyTestName"
    contacts.primaryNameAlt = "PrimaryTestNameAlt"
    contacts.primaryEmail = "pythontest@mail.com"
    contacts.primaryEmailAlt = "pytontestalt@mail.com"
    contacts.primaryPhone = "123456789"
    contacts.primaryPhoneExt = "1234"
    contacts.primaryPhoneAlt = "primaryPhoneAlt"
    contacts.primaryPhoneAltExt = "primaryPhoneAltExt"

    contacts.secundaryName = "SecundaryTestName"
    contacts.secundaryNameAlt = "SecundaryTestNameAlt"
    contacts.secundaryEmail = "pythontest2@mail.com"
    contacts.secundaryEmailAlt = "pytontestalt2@mail.com"
    contacts.secundaryPhone = "012345678"
    contacts.secundaryPhoneExt = "0123"
    contacts.secundaryPhoneAlt = "secundaryPhoneAlt"
    contacts.secundaryPhoneAltExt = "secundaryPhoneAltExt"

    csESAUrl = readESAUrl.getEsaUrl()
    ApplicationUtils().tracelogger().info("esaUrl = {0}".format(csESAUrl))

    print(UpdateSettings().updateContact(csESAUrl, "7e002343bd5d33c9d0e1c890db4ce887", contacts))
