# IBM_PROLOG_BEGIN_TAG
#
# Esaclient 1.0.0
#
# Licensed Materials - Property of IBM
#
# Restricted Materials of IBM
#
# COPYRIGHT International Business Machines Corp. 2014,2017
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# IBM_PROLOG_END_TAG
'''
Created on Apr 16, 2014

@author: Chen Shao Jie
'''
# -*- coding: utf-8 -*-
import ApplicationConstants
from ApplicationUtils import ApplicationUtils
import sys, os, time, atexit
from signal import SIGTERM
from clientprofman import startMonitors



class Daemon:
    """
    A generic daemon class.

    Usage: subclass the Daemon class and override the _run() method
    """
    def __init__(self, pidfile, stdin='/dev/null', stdout='/dev/null', stderr='/dev/null'):
        self.stdin = stdin
        self.stdout = stdout
        self.stderr = stderr
        self.pidfile = pidfile

    def _daemonize(self):

        ApplicationUtils().tracelogger().debug("creating  a daemon process"+self.pidfile)

        # Do first fork.
        try:
            pid = os.fork()
            if pid > 0:
                sys.exit(0)
        except OSError as e:

            sys.stderr.write("fork #1 failed: %d (%s)\n" % (e.errno, e.strerror))
            sys.exit(1)

        ApplicationUtils().tracelogger().debug("Created the first fork process %s daemon()-------------------------------------------------------->"% os.getpid())

        # Decouple from parent environment.
        os.chdir("/")
        os.setsid()
        os.umask(0)

        # Do second fork.
        try:
            pid = os.fork()
            if pid > 0:
                sys.exit(0)
        except OSError as e:
            sys.stderr.write("fork #2 failed: %d (%s)\n" % (e.errno, e.strerror))
            sys.exit(1)

        sys.stdout.flush()
        sys.stderr.flush()
        si = open(os.devnull, 'r')
        so = open(os.devnull, 'a+')
        se = open(os.devnull, 'a+')

        os.dup2(si.fileno(), sys.stdin.fileno())
        os.dup2(so.fileno(), sys.stdout.fileno())
        os.dup2(se.fileno(), sys.stderr.fileno())

        ApplicationUtils().tracelogger().debug("Created the second fork process  %s daemon()-------------------------------------------------------->"%os.getpid())

        # register pid
        atexit.register(self.delpid)
        pid = str(os.getpid())
        fo = open(self.pidfile, 'w+')
        fo.write("%s\n" % pid)

        ApplicationUtils().tracelogger().debug("Created the daemon successfully-------------------------------------------------------->")

    def delpid(self):
        ApplicationUtils().tracelogger().info("Client is shutting down!!!!")
        os.remove(self.pidfile)

    def start(self):
        """
        Start the daemon
        """
        if (self.getstatus() == ApplicationConstants.SEP_RUNNING) :
            sys.stderr.write(ApplicationConstants.SEP_RUNNING)
        else :
            os.system(ApplicationConstants.ENABLE_SERVICE)
            os.system(ApplicationConstants.START_SERVICE)
            sys.stderr.write(ApplicationConstants.SEP_STARTED)
            self.startprovider()

    def startprovider(self):
        ApplicationUtils().tracelogger().info("start()-------------------------------------------------------->")

        try:
            pf = open(self.pidfile, 'r')
            fpid = int(pf.read().strip())
            pf.close()
        except IOError:
            fpid = None

        if fpid:
            pid = self._getpid(fpid)
            if pid != 0:
                message = "Esa client is already running\n"
                sys.stderr.write(message)
                sys.exit(0)


        self._daemonize()
        ApplicationUtils().tracelogger().debug("Created the daemon successfully and initiating run-------------------------------------------------------->")
        self._run()

    def autostart(self):
        """
        Start the daemon
        """
        if (self.getstatus() == ApplicationConstants.SEP_NOT_RUNNING) :
              ApplicationUtils().tracelogger().debug("Created the daemon successfully and initiating run1111-------------------------------------------------------->")

              if os.path.exists(self.pidfile):
                  os.remove(self.pidfile)
              self.startprovider()


    def autostop(self):
        """
        Stop the daemon
        auto stop would be called in the upgrade scenario/reboot scenario where the PID file will not be deleted
        """
        if (self.stopagentandremovepidbytype('None') == ApplicationConstants.STOPPED) :
            sys.stderr.write(ApplicationConstants.STOPPED)
            ApplicationUtils().tracelogger().info(ApplicationConstants.STOPPED)
        else:
            message = ApplicationConstants.SEP_NOT_RUNNING
            sys.stderr.write(message)

    def stop(self):
        """
        Stops the daemon
        """
        self.stopagentandremovepidbytype('None')
        os.system(ApplicationConstants.STOP_SERVICE);
        self.rmsymlinks()
        sys.stderr.write(ApplicationConstants.STOPPED)
        sys.exit(0)

    def stoponuninstall(self):
        """
        Stops the daemon
        """
        self.stopagentandremovepidbytype('None')
        os.system(ApplicationConstants.STOP_SERVICE);
        self.rmsymlinks()
        sys.exit(0)

    def stopagentandremovepidbytype(self,type='None'):
        """
        Stop the daemon
        """
        ApplicationUtils().tracelogger().info("stop()-------------------------------------------------------->")
        try:
            pf = open(self.pidfile, 'r')
            fpid = int(pf.read().strip())
            pf.close()
        except IOError:
            fpid = None

        if fpid:
            pid_list = self._getpidall(fpid)
            for pid in pid_list:
                self.pidkill(int(pid))
            if type=='None' :
                if os.path.exists(self.pidfile):
                    os.remove(self.pidfile)
            return ApplicationConstants.STOPPED
        else:
           return ApplicationConstants.SEP_NOT_RUNNING

    def pidkill(self,pid):
        # Try killing the daemon process
        try:
            while 1:
                os.kill(pid, SIGTERM)
                time.sleep(0.1)
        except OSError as err:
            err = str(err)
            if err.find("No such process") > 0:
                return ApplicationConstants.STOPPED
            else:
                print(err)
                sys.exit(1)

    def rmsymlinks(self):
        os.system(ApplicationConstants.DISABLE_SERVICE);

    def restart(self):
        """
        Restart the daemon
        """
        os.system(ApplicationConstants.STOP_SERVICE);
        os.system(ApplicationConstants.ENABLE_SERVICE);
        os.system(ApplicationConstants.START_SERVICE);
        sys.stderr.write(ApplicationConstants.SEP_RESTARTED)

    def autorestart(self):
        """
        Restart the daemon
        """
        self.stopagentandremovepidbytype()
        self.autostart()

    def getstatus(self):
         # Check for a pidfile to see if the daemon already runs
        try:

            pf = open(self.pidfile, 'r')
            fpid = int(pf.read().strip())
            pf.close()
        except IOError:
            fpid = None

        if fpid:
            pid = self._getpid(fpid)
            if pid != 0:
                return ApplicationConstants.SEP_RUNNING
        return ApplicationConstants.SEP_NOT_RUNNING

    def status(self):
        """
        Check status of the daemon
        """
        # Check for a pidfile to see if the daemon already runs
        if (self.getstatus()== ApplicationConstants.SEP_RUNNING):
                sys.stderr.write(ApplicationConstants.SEP_RUNNING)
                sys.exit(0)
        else:
            sys.stderr.write(ApplicationConstants.SEP_NOT_RUNNING)
            sys.exit(10)

    def _getpid(self, fpid):
        cmd = "ps -ef|grep '" + str(fpid) + "' |grep -v 'grep' |awk '{print $2}' 2>/dev/null"
        ret_text_list = os.popen(cmd)
        pid_list = []
        for line in ret_text_list:
            pid_list.append(line)
        if len(pid_list) > 0:
            pid_num = pid_list[0]
            return int(pid_num)
        else:
            return 0

    def _getpidall(self, fpid):
        cmd = "ps -ef|grep '" + str(fpid) + "' |grep -v 'grep' |awk '{print $2}' 2>/dev/null"
        ret_text_list = os.popen(cmd)
        pid_list = []
        for line in ret_text_list:
            pid_list.append(line)

        return pid_list

    def sysreboot(self):
        # Check for a pidfile to see if the daemon already runs
        try:
            pf = open(self.pidfile, 'r')
            fpid = int(pf.read().strip())
            pf.close()
        except IOError:
            fpid = None

        if fpid:
            self.start()
        else:
            pass

    def _run(self):
        """
        You should override this method when you subclass Daemon. It will be called after the process has been
        daemonized by start() or restart().
        """
        ApplicationUtils().tracelogger().info("inside run initiating run-------------------------------------------------------->")
        raise NotImplementedError
    def checkNetSnmp(self):
        rpm_str = os.popen('rpm -qa net-snmp* --qf "%{name}\n"').read()
        msg = ""
        if rpm_str.find("net-snmp\n") == -1:
            msg = "net-snmp"
            return msg
        if rpm_str.find("net-snmp-libs\n") == -1:
            msg = "net-snmp-libs"
            return msg
        if rpm_str.find("net-snmp-utils\n") == -1:
            msg = "net-snmp-utils"
            return msg
        return msg

    def startSnmpd(self):
        ret = os.system("service snmpd start")
        return ret

    def stopSnmpd(self):
        ret = os.system("service snmpd stop")
        return ret

class ESAClientDaemon(Daemon):
    def _run(self):
        try:
            ApplicationUtils().tracelogger().debug("Calling the method startMonitors")
            ApplicationUtils().tracelogger().debug("Callings the method startMonitors!!!!")
            startMonitors()
        except Exception as e:
            sys.stderr.write("Error in starting EsaClient %s",str(e))
            ApplicationUtils().tracelogger().info("Error in ESAClientDaemon %s",str(e))



if __name__ == "__main__":

    daemon = ESAClientDaemon(ApplicationConstants.AGENT_PID_FILE)
    if len(sys.argv) == 2:
        if 'start' == sys.argv[1]:
            daemon.start()
        elif 'stop' == sys.argv[1]:
            daemon.stop()
        elif 'restart' == sys.argv[1]:
            daemon.restart()
        elif 'status' == sys.argv[1]:
            daemon.status()
        elif 'autostart' == sys.argv[1]:
            daemon.autostart()
        elif 'autostop' == sys.argv[1]:
            daemon.autostop()
        elif 'autorestart' == sys.argv[1]:
            daemon.autorestart()
        elif 'stoponuninstall' == sys.argv[1]:
            daemon.stoponuninstall()
        elif 'getstatus' == sys.argv[1]:
            daemon.getstatus()
        elif '-k'==sys.argv[1] or '-h'==sys.argv[1] or '--help'==sys.argv[1]:
            print("usage: sepcli start|stop|restart|status")
        else:
            print("Unknown parameter")
            print("usage: esacli start|stop|restart|status")
            sys.exit(2)
        sys.exit(0)
    else:
        print("Unknown parameter")
        print("usage: esaclient start|stop|restart|status")
        sys.exit(2)
