# IBM_PROLOG_BEGIN_TAG
#
# serviceable.event.provider 1.0.0
#
# Licensed Materials - Property of IBM
#
# Restricted Materials of IBM
#
# COPYRIGHT International Business Machines Corp. 2014,2014
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# IBM_PROLOG_END_TAG
'''
Created on Oct 03, 2016

@author: Ashutosh Agarwal
'''

import sys
import socket
import ssl
import struct
import threading
import json
import urllib.request
import copy
import ApplicationConstants
import ApplicationNotify
from ApplicationUtils import ApplicationUtils
import SynchronizConf

class _receiveMulticastResponseThread(threading.Thread):
    def __init__(self,responseList):
        threading.Thread.__init__(self)
        self._responseList = responseList

    def run(self):
        # Create a UDP socket
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        try:
            portNo = int(ApplicationConstants.RECEIVER_PORT)
            # handle empty or wrong RECEIVER_PORT
            server_address = ('', portNo)

            # awaits seconds for the client to connect, this can be configurable
            sock.settimeout(int(ApplicationConstants.RECEIVER_TIMEOUT))
            sock.bind(server_address)
            # closes the connection on the first timeout
            timeOut=True

            while timeOut:
                try:
                    ApplicationUtils().tracelogger().info("awaiting to receive responses from client at {0}".format(portNo))
                    data, server = sock.recvfrom(4096)
                    ApplicationUtils().tracelogger().info("Received response from {0} and data is {1}".format(server, data))
                    dataJson = json.loads(data)
                    self._responseList.append(dataJson)
                    #print >>sys.stderr, 'json %s' % dataJson.port
                except socket.timeout as e:
                    ApplicationUtils().tracelogger().info("Logical Socket Timeout ({0}): {1}".format(e.errno, e.strerror))
                    timeOut=False
            #threading.Thread(target = self.listenToClient,args = (client,address)).start()
        except socket.error as e:
            ApplicationUtils().tracelogger().error("Socket Error({0}): {1}".format(e.errno, e.strerror))
            print ("Receiving Address configuration Error ({0}): {1}".format(e.errno, e.strerror))
        except ValueError:
            print ("Bad Configuration of Receiver Port No: {0} ".format(ApplicationConstants.RECEIVER_PORT))

        #except Exception as ex:
            #print "Global Exception ({0}): {1}".format(ex.errno, ex.strerror)
        finally:
            ApplicationUtils().tracelogger().info("closing socket")
            sock.close()

def sendMulticast():
    # Create the datagram socket
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    # Set a timeout so the socket does not block indefinitely when trying to receive data.
    sock.settimeout(0.2)
    # Set the time-to-live for messages to 1 so they do not go past the
    # local network segment.
    ttl = struct.pack('b', 1)
    sock.setsockopt(socket.IPPROTO_IP, socket.IP_MULTICAST_TTL, ttl)

    try:
        # message sent to csesa multicast address
        message = ApplicationConstants.RECEIVER_PORT
        multicast_group = (ApplicationConstants.SERVER_MULTICAST_ADDRESS, int(ApplicationConstants.SERVER_MULTICAST_PORT))
        # Send data to the multicast group
        ApplicationUtils().tracelogger().info("locateESA sendMulticast sending {0}".format(message))
        sock.sendto(message.encode(), multicast_group)

    except socket.error as e:
        ApplicationUtils().tracelogger().error("sendMulticast Socket Error({0}): {1}".format(e.errno, e.strerror))
        exp = Exception("Bad Configuration of Multicast Address: {0}".format(ApplicationConstants.SERVER_MULTICAST_ADDRESS))
        exp.errno = e.errno
        raise exp
    except ValueError:
            print ("Bad Configuration of Multicast Port No: {0} ".format(ApplicationConstants.SERVER_MULTICAST_PORT))
    finally:
        #appUtils().info('locateESA sendMulticast closing socket')
        sock.close()

def getCsESAUrls():
    # csESAVersion is temp hack mentioning the version of the webservice. Will update after Oct 15
    csESAVersion=ApplicationConstants.CSESA_VERSION
    responseList=[]
    thread = _receiveMulticastResponseThread(responseList)
    thread.start()

    sendMulticast()

    thread.join()

    ApplicationUtils().tracelogger().info("Before responseList is {0}".format(json.dumps(responseList, sort_keys=True,indent=4, separators=(',', ': '))))

    finalResponseList=[]

    ctx = ssl.create_default_context()
    ctx.check_hostname = False
    ctx.verify_mode = ssl.CERT_NONE

    for response in responseList:
        ApplicationUtils().tracelogger().info("esa-role is {0}".format(response['esa-role']))
        ApplicationUtils().tracelogger().info("esa-server-name is {0}".format(response['esa-server-name']))
        esaURLs=[]

        for aUrl in response['esa-urls']:
            ApplicationUtils().tracelogger().info("esa-urls is {0}".format(aUrl['url']))
            validURL = False
            try:
                httpResponse = urllib.request.urlopen(aUrl['url'], data=None, timeout=3, context=ctx)
                #httpCode = httpResponse.getcode()
                ApplicationUtils().tracelogger().info("httpResponseCode is {0}".format(httpResponse.getcode()))
                #ApplicationUtils().tracelogger().info("httpResponseContent is {0}".format(httpResponse.read))
                if httpResponse.getcode() == 200:
                    # success
                    #if httpResponse.info().gettype() == 'application/json':
                        dataJson = json.loads(httpResponse.read().decode("utf-8"))
                        #ApplicationUtils().tracelogger().info("httpResponseRead is {0}".format(json.dumps(dataJson, sort_keys=True,indent=4, separators=(',', ': '))))
                        if dataJson['status']['code'] == 200:
                            validURL = True
                            #print csESAVersion

            except urllib.request.HTTPError as he:
                ApplicationUtils().tracelogger().error("urllib2.HTTPError ({0}): {1}".format(he.errno, he.strerror))
            except urllib.request.URLError as ue:
                ApplicationUtils().tracelogger().error("urllib2.URLError ({0}): {1}".format(ue.errno, ue.strerror))
            except BaseException as be:
                ApplicationUtils().tracelogger().error("BaseException {0}".format(be))

            if validURL:
                #add
                copiedURL = copy.copy(aUrl)
                copiedURL['url']=copiedURL['url']
                esaURLs.append(copiedURL)
                ApplicationUtils().tracelogger().info("changedUrl is {0}".format(copiedURL['url']))

        #outer for loop
        if esaURLs:
            #add
            copiedResponse = copy.copy(response)
            # convert httpResponse body to json and then use esa-api-version from the object and append to the url. find what happens
            copiedResponse['esa-urls']=esaURLs
            finalResponseList.append(copiedResponse)

    #httpResponse = urllib2.urlopen(url).read()
    #for ends

    return finalResponseList


if __name__ == "__main__":

#   it ensures that the log config file is changed so that the log file is created at the appropriate place
    SynchronizConf.synchronizConf()
    try:
        csESAUrls = getCsESAUrls()
        csESAUrlsString = json.dumps(csESAUrls, sort_keys=True,indent=4, separators=(',', ': '))
        ApplicationUtils().tracelogger().info("After filtering finalResponseList is {0}".format(csESAUrlsString))
        print(csESAUrlsString)
    except Exception as be:
        print ("{0}".format(str(be)))
    sys.exit(0)


