# IBM_PROLOG_BEGIN_TAG
#
# Esaclient 1.0.0
#
# Licensed Materials - Property of IBM
#
# Restricted Materials of IBM
#
# COPYRIGHT International Business Machines Corp. 2014,2017
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# IBM_PROLOG_END_TAG



'''
@author: Madhavi Rani
'''
import threading
from threading import Timer
import datetime
import ApplicationUtils
import ApplicationConstants
import re
from time import sleep




class Schedular:

    def __init__(self,function,*args, **kwargs):

        self._function = function
        self.args       = args
        self.kwargs     = kwargs

    def schedule(self):
        """
        You should override this method when you subclass Schedular. """
        raise NotImplementedError


#This reads the pattern specified in the args and executes the function based on
#the value of the pattern
#?where to put this value of the pattern
#Write the pattern that would be parsed here

#Description/Function/Command, C=Continuous/S=Scheduled, Interval (m/h/d/w), Time (24 :00)
#problemProvider=C,
#cisaInentoryA=S, 3w, 3:33
#plinuxPM=S, 1d, 3:44
#heartbeat=S, 8h,
#If the command is not present it is considered as disabled
class ESASchedular(Schedular):


    def scheduleAt(self,timeString,repeatingjob):

        if timeString is not None and timeString:
            repeatingjob = repeatingjob.at(timeString)

        return repeatingjob

    def schedule(self):
        """
        You should override this method when you subclass Schedular. """


        apputils = ApplicationUtils.ApplicationUtils()
        apputils.tracelogger().debug("Inside the schedule function")

        try:
            value = self._patternParser()
        except Exception as e:
            apputils.tracelogger().debug("Error in parsing the pattern %s"%(e))
            return


        if value is None:
            apputils.tracelogger().debug("pattern parser returned a null value hence not proceeding further")
            return


        #check if it is ok to use this representation
        #or should this be changed


        isScheduleJob=value[ApplicationConstants.SCHEDULE_JOB_TYPE]
        apputils.tracelogger().debug("Schedule pattern %s"%(isScheduleJob.strip()))




        if isScheduleJob.strip() == 'S' or isScheduleJob.strip() == 's':

            apputils.tracelogger().debug("This is a scheduled Job hence calling scheduled job class")

            timeString=value[ApplicationConstants.SCHEDULE_TIME_STRING]
            intervalString=value[ApplicationConstants.SCHEDULE_INTERVAL_STRING]

            intervalString = intervalString.strip();
            interval = intervalString[:-1]
            units = intervalString[-1:]

            apputils.tracelogger().debug("intervalString %s"%(intervalString))
            apputils.tracelogger().debug("interval %s"%(interval))
            apputils.tracelogger().debug("units %s"%(units))
            apputils.tracelogger().debug("timeString %s"%(timeString))

            repeatingJob = RepeatingJob(True,self._function)

            if  units == 'd' or units == 'D':

                repeatingJob = repeatingJob.scheduleDaily(interval).at(timeString)

            if  units == 'w' or units == 'W':

                repeatingJob = repeatingJob.scheduleWeekly(interval).at(timeString)

            if units == 'm' or units =='M' :

                repeatingJob = repeatingJob.scheduleMin(interval)

            if units == 's' or units == 'S':

                repeatingJob = repeatingJob.scheduleSec(interval)


            if units =='h' or units == 'H':

                repeatingJob = repeatingJob.scheduleHourly(interval);
            repeatingJob.start()
        elif isScheduleJob.strip() == 'C' or isScheduleJob.strip() == 'c':

            apputils.tracelogger().debug("This is a continuous Job , creating a thread to execute the specified function %s"%(self._function))
            continuousJob=ContinuousJob(self._function)
            continuousJob.start()
            apputils.tracelogger().debug("started the continuous job")
        else:
            apputils.tracelogger().error("Error in schedule pattern.Not Processing anything")




    def _validateSchedulePattern(self,schedulePattern):

        return  re.match(ApplicationConstants.SCHEDULE_PATTERN_REGEX, schedulePattern)

    def _patternParser(self):

        keyToLookInConfFile = self.kwargs[ApplicationConstants.PROVIDER_KEY]


        apputils = ApplicationUtils.ApplicationUtils()


        try:

            schedulePattern = ApplicationUtils.parseConfigForKey(ApplicationConstants.DAEMON_SCHEDULE, keyToLookInConfFile, None)
            apputils.tracelogger().debug("parsing the pattern %s"%(schedulePattern))
            timeString=None
            intervalString=None

            if schedulePattern is None:
                apputils.tracelogger().info("service  %s is not enabled.Hence not starting the service"%(keyToLookInConfFile))
                return

            isPatternValid=self._validateSchedulePattern(schedulePattern)

            if isPatternValid is None:
                apputils.tracelogger().info("schedule pattern  %s is not valid.Hence not starting the service"%(schedulePattern))
                apputils.tracelogger().warn("provider with %s is not started.If this is the only provider configured, this might bring down the client"%(keyToLookInConfFile))
                return

            #split the pattern by comma separator and parse the values for job type,interval string and time string
            schedTokens = schedulePattern.split(ApplicationConstants.SCHEDULE_PATTERN_DELIMITER)
            tokenLength = len(schedTokens)
            isScheduleJob = schedTokens[0]

            if isScheduleJob.strip() == 'S' or isScheduleJob.strip() == 's':
                intervalString = schedTokens[1]
                if tokenLength == 3:
                    timeString=schedTokens[2]


            return {ApplicationConstants.SCHEDULE_JOB_TYPE:isScheduleJob,
                    ApplicationConstants.SCHEDULE_INTERVAL_STRING:intervalString,
                    ApplicationConstants.SCHEDULE_TIME_STRING:timeString}

        except Exception as e:
            apputils.tracelogger().error("Error in parsing schedule pattern %s",str(e))

class ContinuousJob(threading.Thread):

    def __init__(self,function,jobtype=ApplicationConstants.CONTINUOUS_JOB_TYPE_ONE_TIME,*args,**kwargs):

        self.function=function
        self.jobtype=jobtype
        self.args=args
        self.kwargs=kwargs
        threading.Thread.__init__(self)

    #recurring continuous job is not yet supported.for now time interval is mentioned as 10 min

    def run(self):

        if self.jobtype==ApplicationConstants.CONTINUOUS_JOB_TYPE_ONE_TIME:
            ApplicationUtils.ApplicationUtils().tracelogger().debug("Executing the function %s as a one time job"%self.function)
            self.function(*self.args, **self.kwargs)
        elif self.jobtype==ApplicationConstants.CONTINUOUS_JOB_TYPE_RECURRING:
            self.runRecurringJob()



    def runRecurringJob(self):

        while True:

            self.function(*self.args, **self.kwargs)
            sleep(600)


class RepeatingJob:

    def __init__(self,startnow,function, *args, **kwargs):

        self._timer     = None
        self.function   = function
        self.interval   = 24*60*60
        self._intervalInSecs = 24*60*60
        self._unit      = "seconds"
        self.startnow   = startnow
        self.args       = args
        self.kwargs     = kwargs
        self._at_hour = None
        self._at_min  = None


    def _run(self):

        self.start()
        self.function(*self.args, **self.kwargs)

    def start(self):
        #if not self.is_running:

            self. _calculateNextScheduleDate()
            self._timer = Timer(self._intervalInSecs, self._run)
            self._timer.start()
            if(self.startnow):
                self.function(*self.args, **self.kwargs)
                self.startnow=False

            #self.is_running = True

    def scheduleMin(self,interval):

        self.interval = interval
        self._unit = "minutes"
        return self

    def scheduleSec(self,interval):

        self.interval = interval
        self._unit = "seconds"
        return self

    def scheduleHourly(self,interval):


        self.interval = interval
        self._unit="hours"
        return self

    def scheduleDaily(self,interval):

        self.interval=interval
        self._unit="days"
        return self

    def scheduleWeekly(self,interval):

        self.interval=interval
        self._unit="weeks"
        return self

    def at(self,timeString):

        if timeString is not None and timeString:

            self._at_hour,self._at_min = timeString.split(":")
            self.startnow=False

        return self


    def _calculateNextScheduleDate(self):

        apputils = ApplicationUtils.ApplicationUtils()
        apputils.tracelogger().debug("Calculating the next schedule date")
        curDay = datetime.datetime.now()
        apputils.tracelogger().debug("current time is %s"%curDay)

        currDateString = curDay.strftime('%d-%m-%Y %H-%M-%S')
        currDate = curDay.strptime(currDateString,'%d-%m-%Y %H-%M-%S')

        interval_type = self._unit
        interval_num = self.interval

        apputils.tracelogger().debug("Interval type is %s,Interval value is %s"%(interval_type,interval_num))

        nextDay = datetime.datetime.now() + datetime.timedelta(**{interval_type: int(interval_num)})

        nextDateString = nextDay.strftime('%d-%m-%Y %H-%M-%S')

        newDate = nextDay.strptime(nextDateString,'%d-%m-%Y %H-%M-%S')

        if self._at_hour is not None and  self._at_min is not None:

            newDate = newDate.replace(hour=int(self._at_hour), minute=int(self._at_min))

        apputils.tracelogger().debug("New date is  %s"%(newDate))

        timediff = (newDate-currDate).total_seconds()

        apputils.tracelogger().debug("Time diff is  %s"%(timediff))

        self._intervalInSecs=timediff




    def getSchedule(self,iteration):

        #Give the schedule as list
        print("schedule to be implemented")





    def stop(self):
        self._timer.cancel()
        self.is_running = False


