# IBM_PROLOG_BEGIN_TAG
#
# serviceable.event.provider 1.0.0
#
# Licensed Materials - Property of IBM
#
# Restricted Materials of IBM
#
# COPYRIGHT International Business Machines Corp. 2014,2014
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# IBM_PROLOG_END_TAG

import ssl
import json
import urllib.request
from readESAUrl import getEsaUrl
from ApplicationUtils import ApplicationUtils


def sendHeartbeatInit():
    systemId = ApplicationUtils().getSystemId("primary")       
    if systemId=="":
        ApplicationUtils().tracelogger().info("Cannot get systemId systemid file")
        return 
    
    restUrl = readRestUrl() 
    if restUrl=="":
        ApplicationUtils().tracelogger().info("Cannot get rest url")
        return 
    
    sendHeartbeat(systemId, restUrl)
    
def readRestUrl():    
    restUrl=""
    try:
        restUrl = getEsaUrl()
    except Exception:
        print("An error happened when getting rest url")
        ApplicationUtils().tracelogger().error("An error happened when getting rest url")
    print("restUrl="+ restUrl)    
    return restUrl   
   
def sendHeartbeat(systemId, temUrl):
    try:
        csESAUrl = temUrl + '/data/hb/' + systemId
        values = {'endpoints':[]}
        data = json.dumps(values).encode()
        
        h = {"Content-Type": "application/json"}
        req = urllib.request.Request(csESAUrl, data, headers=h)
        req.get_method = lambda: 'POST'
        
        ctx = ssl.create_default_context()
        ctx.check_hostname = False
        ctx.verify_mode = ssl.CERT_NONE
        
        httpResponse = urllib.request.urlopen(req, context=ctx)

        print (httpResponse.geturl())
        ApplicationUtils().tracelogger().info("httpResponseCode is {0}".format(httpResponse.getcode()))
        
        if httpResponse.getcode() == 200:
            dataJson = json.loads(httpResponse.read().decode("utf-8"))
            code = dataJson['status']['code']
            if (code == 200) or (code == 201):
                result = dataJson['details']
                print(result)
                ApplicationUtils().tracelogger().info(result)
            elif (code == 451) or (code == 404):
                result = dataJson['status']['message']
                print(result)
                ApplicationUtils().tracelogger().info(result)
    except urllib.request.HTTPError as he:
        ApplicationUtils().tracelogger().error("An error happened when call send Heartheat API.")
        if hasattr(he, 'code'):
            print('Error code: ', str(he.code))
            ApplicationUtils().tracelogger().error('Error code: ', str(he.code))
    except urllib.request.URLError as ue:
        ApplicationUtils().tracelogger().error("An error happened when call send Heartheat API.")
        if hasattr(ue,"reason"):
            print("Reason:"+ str(ue.reason))
        ApplicationUtils().tracelogger().error("Reason:"+ str(ue.reason))
        if hasattr(ue, 'code'):
            print('Error code: ', str(ue.code))
            ApplicationUtils().tracelogger().error('Error code: '+ str(ue.code))
    except BaseException as be:
        print("An error happened when call send Heartheat API.BaseException %s" %be)
        ApplicationUtils().tracelogger().error("An error happened when call send Heartheat API.BaseException {0}".format(be))
       
    return

if __name__ == "__main__":
    sendHeartbeatInit()
