# IBM_PROLOG_BEGIN_TAG
#
# serviceable.event.provider 1.0.0 
#
# Licensed Materials - Property of IBM
#
# Restricted Materials of IBM
#
# COPYRIGHT International Business Machines Corp. 2014,2014
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# IBM_PROLOG_END_TAG
'''
Created on Apr 16, 2014

@author: Chen Shao Jie
'''
# -*- coding: utf-8 -*-
import sys, os, time, atexit,ApplicationNotify
from signal import SIGTERM
import ApplicationConstants
import ApplicationUtils
import StartAgent

class Daemon:
    """
    A generic daemon class.
    
    Usage: subclass the Daemon class and override the _run() method
    """
    def __init__(self, pidfile, stdin='/dev/null', stdout='/dev/null', stderr='/dev/null'):
        self.stdin = stdin
        self.stdout = stdout
        self.stderr = stderr
        self.pidfile = pidfile

    def _daemonize(self):
        
        # Do first fork.
        try:
            pid = os.fork()
            if pid > 0:
                sys.exit(0)
        except OSError, e:
            sys.stderr.write("fork #1 failed: %d (%s)\n" % (e.errno, e.strerror))
            sys.exit(1)

        # Decouple from parent environment.  
        os.chdir("/")
        os.setsid()
        os.umask(0)

        # Do second fork.  
        try:
            pid = os.fork()
            if pid > 0:
                sys.exit(0)
        except OSError, e:
            sys.stderr.write("fork #2 failed: %d (%s)\n" % (e.errno, e.strerror))
            sys.exit(1)

        sys.stdout.flush()
        sys.stderr.flush()
        si = file(self.stdin, 'r')
        so = file(self.stdout, 'a+')
        se = file(self.stderr, 'a+', 0)
        
        # Redirect standard file descriptors.
        os.dup2(si.fileno(), sys.stdin.fileno())
        os.dup2(so.fileno(), sys.stdout.fileno())
        os.dup2(se.fileno(), sys.stderr.fileno())

        # register pid
        atexit.register(self.delpid)
        pid = str(os.getpid())
        file(self.pidfile, 'w+').write("%s\n" % pid)

    def delpid(self):
        os.remove(self.pidfile)
        
    def start(self):
        """
        Start the daemon
        """
        os.system(ApplicationConstants.ENABLE_SERVICE);
        os.system(ApplicationConstants.START_SERVICE);
        #self.startprovider(type)
        
    def startprovider(self):         
        ApplicationUtils.ApplicationUtils().tracelogger().info("start()-------------------------------------------------------->")
        try:
            pf = file(self.pidfile, 'r')
            fpid = int(pf.read().strip())
            pf.close()
        except IOError:
            fpid = None
            
        if fpid:
            pid = self._getpid(fpid)
            if pid != 0:
                message = "serviceable event provider is already running\n"
                sys.stderr.write(message)
                sys.exit(0)
        
        sys.stderr.write("serviceable event provider started successfully\n")
        # Start the daemon
        self._daemonize()
        self._run()
        
    def autostart(self):
        """
        Start the daemon
        """
        if (self.getstatus() == ApplicationConstants.SEP_NOT_RUNNING) :
              if os.path.exists(self.pidfile):
                  os.remove(self.pidfile)
              self.startprovider()
                    
    def autostop(self):
        """
        Stop the daemon
        auto stop would be called in the upgrade scenario/reboot scenario where the PID file will not be deleted
        """
        if (self.stopagentandremovepidbytype('None') == ApplicationConstants.STOPPED) :
            sys.stderr.write(ApplicationConstants.STOPPED)
            ApplicationUtils.ApplicationUtils().tracelogger().info(ApplicationConstants.STOPPED)
        else:
            message = ApplicationConstants.SEP_NOT_RUNNING
            sys.stderr.write(message)
                     
    def stop(self):
        """
        Stops the daemon
        """
        self.rmsymlinks()
        os.system(ApplicationConstants.STOP_SERVICE);
        sys.exit(10)
            
    def stoponuninstall(self):
        """
        Stops the daemon
        """
        os.system(ApplicationConstants.STOP_SERVICE);
        sys.exit(0)
            
    def stopagentandremovepidbytype(self,type='None'):   
        """
        Stop the daemon
        """
        ApplicationUtils.ApplicationUtils().tracelogger().info("stop()-------------------------------------------------------->")
        try:
            pf = file(self.pidfile, 'r')
            fpid = int(pf.read().strip())
            pf.close()
        except IOError:
            fpid = None
       
        if fpid:
            # Try killing the daemon process    
            try:
                while 1:
                    os.kill(fpid, SIGTERM)
                    time.sleep(0.1)
            except OSError, err:
                err = str(err)
                if err.find("No such process") > 0:
                    if type=='None' :
                        if os.path.exists(self.pidfile):
                            os.remove(self.pidfile)
                    return ApplicationConstants.STOPPED
                else:
                    print str(err)
                    sys.exit(1)
        else:
           return ApplicationConstants.SEP_NOT_RUNNING
          
                 
    def rmsymlinks(self):
        os.system(ApplicationConstants.DISABLE_SERVICE);    
       
    def restart(self):
        """
        Restart the daemon
        """
        os.system(ApplicationConstants.STOP_SERVICE); 
        os.system(ApplicationConstants.ENABLE_SERVICE);
        os.system(ApplicationConstants.START_SERVICE);
        
    def autorestart(self):
        """
        Restart the daemon
        """
        self.stopagentandremovepidbytype()
        self.autostart()
        
    def getstatus(self): 
         # Check for a pidfile to see if the daemon already runs
        try:
            pf = file(self.pidfile, 'r')
            fpid = int(pf.read().strip())
            pf.close()
        except IOError:
            fpid = None
            
        if fpid:
            pid = self._getpid(fpid)
            if pid != 0:
                return ApplicationConstants.SEP_RUNNING
        return ApplicationConstants.SEP_NOT_RUNNING
        
    def status(self):
        """
        Check status of the daemon
        """
        # Check for a pidfile to see if the daemon already runs
        if (self.getstatus()== ApplicationConstants.SEP_RUNNING):
                sys.stderr.write(ApplicationConstants.SEP_RUNNING)
                sys.exit(0)
        else:         
            sys.stderr.write(ApplicationConstants.SEP_NOT_RUNNING)
            sys.exit(10)

    def _getpid(self, fpid):
        cmd = "ps -ef|grep '" + str(fpid) + "' |grep -v 'grep' |awk '{print $2}' 2>/dev/null"      
        ret_text_list = os.popen(cmd)  
        pid_list = []  
        for line in ret_text_list:  
            pid_list.append(line)
        if len(pid_list) > 0:
            pid_num = pid_list[0]  
            return int(pid_num)
        else:
            return 0
    
    def sysreboot(self):
        # Check for a pidfile to see if the daemon already runs
        try:
            pf = file(self.pidfile, 'r')
            fpid = int(pf.read().strip())
            pf.close()
        except IOError:
            fpid = None
            
        if fpid:
            self.start()
        else:
            pass
        
    def _run(self):
        """
        You should override this method when you subclass Daemon. It will be called after the process has been
        daemonized by start() or restart().
        """
    
class MyDaemon(Daemon):
    def _run(self):
        StartAgent.start()

if __name__ == "__main__":
    daemon = MyDaemon(ApplicationConstants.AGENT_PID_FILE)
    if len(sys.argv) == 2:
        if 'start' == sys.argv[1]:
            daemon.start()
        elif 'stop' == sys.argv[1]:
            daemon.stop()
        elif 'restart' == sys.argv[1]:
            daemon.restart()
        elif 'status' == sys.argv[1]:
            daemon.status()
        elif 'autostart' == sys.argv[1]:
            daemon.autostart()
        elif 'autostop' == sys.argv[1]:
            daemon.autostop()
        elif 'autorestart' == sys.argv[1]:
            daemon.autorestart()
        elif 'stoponuninstall' == sys.argv[1]:
            daemon.stoponuninstall()
        elif 'getstatus' == sys.argv[1]:
            daemon.getstatus()
        elif '-k'==sys.argv[1] or '-h'==sys.argv[1] or '--help'==sys.argv[1]:
            print "usage: sepcli start|stop|restart|status"
        else:
            print "Unknown parameter"
            print "usage: sepcli start|stop|restart|status"
            sys.exit(2)
        sys.exit(0)
    else:
        print "Unknown parameter"
        print "usage: sepcli start|stop|restart|status"
        sys.exit(2)
